# 
# @(#) $Id: fcpc.py,v 1.18 2002/09/08 15:35:09 ivm Exp $
#
# $Log: fcpc.py,v $
# Revision 1.18  2002/09/08 15:35:09  ivm
# Added select_cfg scripts
#
# Revision 1.17  2001/09/11 14:53:00  ivm
# Fixed long-lasting CPUsage bug
#
# Revision 1.14  2001/09/10 15:15:02  ivm
# Do import time in fcpc.py
#
# Revision 1.13  2001/03/16 23:16:44  ivm
# Added Install.txt
# Fixed bug in FCPClient.listQueue()
#
# Revision 1.12  2000/12/14 16:55:22  ivm
# fixed connect() in client
# add (...) to start_fcp.sh
#
# Revision 1.10  2000/11/21 19:38:29  ivm
# Renamed find to translate
#
# Revision 1.7  2000/11/07 20:48:25  ivm
# Implemented multi-volume placement. Still needs polishing.
#
# Revision 1.6  2000/10/25 22:44:11  ivm
# Tested scheduling and renewal
#
# Revision 1.4  2000/08/14 17:32:29  ivm
# Added fcp.cfg
# Read parameters from fcp.cfg
#
# Revision 1.1  2000/08/14 16:02:01  ivm
# Initial versions
#
#

import string
import sys
import stat
import os
import pwd
import time
from SockStream import SockStream
from config import ConfigFile
from socket import *

class	FCPClient:
	def __init__(self, host, cfg, project = None, user = None):
		self.ServerHost = host
		self.ServerPort = cfg.getValue('fcp','*','port')
		self.Sock = None
		self.Str = None
		self.Username = user
		if not self.Username:
			self.Username = pwd.getpwuid(os.getuid())[0]
		self.RenewInterval = 30
		self.Project = self.Username
		if project:
			self.Project = project
			
	def connect(self, tmo = 300):
		if self.connected():
			return 1
		done = 0
		t = time.time()
		while not done:
			self.Sock = socket(AF_INET, SOCK_STREAM)
			try:	self.Sock.connect((self.ServerHost, self.ServerPort))
			except:
				pass
			else:
				self.Str = SockStream(self.Sock)
				# for now, use username as project name
				ans = self.Str.sendAndRecv('HELLO %s %s %d' % 
						(self.Username, self.Project, os.getpid()))
				if ans:
					done = 1
					break
			if tmo >= 0 and time.time() > t + tmo:
				break
			time.sleep(1)
		return done

	def disconnect(self):
		self.Str = None
		if self.Sock != None:
			self.Sock.close()
		self.Sock = None
		
	def connected(self):
		return self.Sock != None and self.Str != None

	def requestUpload(self, rfn, size, tmo = -1):
		if not self.connect(300):
			return 0, 'ERR Can not connect to fcpd', None
		try:	ans = self.Str.sendAndRecv('UL %s %d' % (rfn, size))
		except IOError, val:
			self.disconnect()
			return 0, 'Server timed-out', None
		if not ans:
			self.disconnect()
			return 0, 'Server disconnected', None
		words = string.split(ans)
		if not words:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		if words[0] == 'ERR':
			self.disconnect()
			return 0, ans, None
		if words[0] == 'WAIT':
			# wait for GRANTED
			try:	ans = self.Str.recv(tmo = tmo)
			except IOError:
				self.disconnect()
				return 0, 'Time-out', None
			if not ans:
				self.disconnect()
				return 0, 'Server disconnected', None
			words = string.split(ans)
		if not words:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		if words[0] == 'ABORTED':
			self.disconnect()
			return 0, 'Request aborted', None
		if words[0] != 'GRANTED' or len(words) < 4:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		self.RenewInterval = string.atoi(words[2])
		return 1, words[1], words[3]

	def requestDownload(self, rfn, tmo = -1):
		if not self.connect(300):
			return 0, 'ERR Can not connect to fcpd', None
		try:	ans = self.Str.sendAndRecv('DL %s' % rfn)
		except IOError:
			self.disconnect()
			return 0, 'Server timed-out', None
		if not ans:
			self.disconnect()
			return 0, 'Server disconnected', None
		words = string.split(ans)
		if not words:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		if words[0] == 'ERR':
			self.disconnect()
			return 0, ans, None
		if words[0] == 'WAIT':
			# wait for GRANTED
			try:	ans = self.Str.recv(tmo = tmo)
			except IOError:
				self.disconnect()
				return 0, 'Time-out', None
			if not ans:
				self.disconnect()
				return 0, 'Server disconnected', None
			words = string.split(ans)
		if not words:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		if words[0] == 'ABORTED':
			self.disconnect()
			return 0, 'Request aborted', None
		if words[0] != 'GRANTED' or len(words) < 4:
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans, None
		self.RenewInterval = string.atoi(words[2])
		return 1, words[1], words[3]

	def finishRequest(self, rid):
		if not self.connect(60):
			return 0, 'ERR Can not connect to fcpd'
		ans = self.Str.sendAndRecv('FIN %s' % rid)
		if not ans:
			self.disconnect()
			return 0, 'Server disconnected'
		if ans != 'OK':
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans
		return 1, 'OK'

	def renewInterval(self):
		return self.RenewInterval
	
	def renewRequest(self, rid):
		if not self.connect(60):
			return 0, 'ERR Can not connect to fcpd'
		ans = self.Str.sendAndRecv('RENEW %s' % rid)
		if not ans:
			self.disconnect()
			return 0, 'Server disconnected'
		if ans != 'OK':
			self.disconnect()
			return 0, 'Protocol error: <%s>' % ans
		return 1, 'OK'
	
	def listQueue(self):
		lst = []
		if not self.connect(5):
			return 0, 'ERR Can not connect to fcpd'
		ans = self.Str.sendAndRecv('LIST')
		while ans and ans != '.':
			words = string.split(ans)
			if len(words) >= 9:
				lst.append(tuple(words[:9]))
			ans = self.Str.recv()
		return 1, lst

	def abortRequest(self, rid):
		if not self.connect(5):
			return 0, 'ERR Can not connect to fcpd'
		ans = self.Str.sendAndRecv('ABORT %s' % rid)
		if ans == 'OK':
			return 1, 'OK'
		else:
			self.disconnect()
			return 0, ans

	def translate(self, vfn):
		if not self.connect(300):
			return 0, 'ERR Can not connect to fcpd'
		ans = self.Str.sendAndRecv('XLATE %s' % vfn)
		lst = string.split(ans)
		if lst:
			if lst[0] == 'NF':
				return 0, 'File <%s> not found' % vfn
			if lst[0] == 'OK':
				return 1, lst[1]
		self.disconnect()
		return 0, 'Protocol error: <%s>' % ans

CPUsage = """Usage:
    fcp [-p <project>] [-t <time-out>] [-v] [-c <command> [<cmdopts>]] 
        <file> [<user>@]<host>:<file>
        or
        [<user>@]<host>:<file> <file>
"""

Usage = """Usage:
Obtain information:
    fcps <host>

Transfer file:
    fcp [-p <project>] [-t <time-out>] [-v] [-c <command> [<cmdopts>]] 
        <file> [<user>@]<host>:<file>
        or
        [<user>@]<host>:<file> <file>

Abort transfer:
    fcp_abort <host> <rid>

Find file by virtual name:
    fcp_translate <host> <virtual file name>
"""

def cmprq(x, y):
	if x[0] < y[0]: return -1
	elif x[0] == y[0]: return 0
	else:	return 1

def doList(argv):
	import getopt
	host = None
	port = None
	args = argv
	if not args:
		print Usage
		return 2
	host = args[0]
	print '%4s %30s %2s %13s %6s %s' % (
		'RID', 'user.pid@host', 'TS', 'Free/Avbl(MB)', 'Size', 
		'File')
	print '%4s %30s %2s %13s %6s %s' % (
		'----', '------------------------------', '--', 
		'-------------', '------', '----------')

	FCPC = FCPClient(host, ConfigFile(os.environ['FCP_CONFIG']))
	try:	sts, ans = FCPC.listQueue()
	except:
		print 'fcpd communication error: %s %s' % (sys.exc_type, sys.exc_value)
		sys.exit(1)
	if not sts:
		print ans
		sys.exit(1)
	qlst = ans		
	qlst.sort(cmprq)
	for id, cid, user, state, t, path, size, free, rsrv in qlst:
		i = string.find(cid, '.')
		if i > 0:
			cid = cid[:i]
		if t == 'D':
			free = ''
			rsrv = ''
			fa = ''
		else:
			try:
				avail = string.atoi(free) - string.atoi(rsrv)
				fa='%6s/%-6s' % (free, avail)
			except:
				avail = 'N/A'
				fa = 'N/A'
		who = '%s.%s' % (user, cid)
		st = t
		if state == 'R':
			st = st + '*'
		else:
			st = st + 'w'
		print '%4s %30s %2s %13s %6s %s' % (id, who,
			st,  
			fa, size, path)
	return 0

def doTransfer(fcpc, rid, cmd):
	renew = fcpc.renewInterval()
	pid = os.fork()
	if pid > 0:
		# parent
		done = 0
		t = time.time()
		while not done:
			pid, sts = os.waitpid(pid, os.WNOHANG)
			if pid > 0:
				done = 1
			else:
				time.sleep(1)
				if time.time() > t + renew:
					fcpc.renewRequest(rid)
					t = time.time()
		fcpc.finishRequest(rid)
		return sts >> 8
	else:
		# child
		#print cmd
		apply(os.execlp, (cmd[0],) + cmd)

def doCopy(argv):
	import getopt
	p_opt = ''
	host = None
	port = None
	verbose = 0
	project = None
	tmo = -1
	i = 0
	cmd = ['rcp']
	cmdopt = []
	if len(argv) < 2:
		print CPUsage
		return 2
	f1 = argv[-2]
	f2 = argv[-1]
	argv = argv[:-2]
	in_cmd = 0
	while i < len(argv):
		if not in_cmd:
			if argv[i] == '-v':
				verbose = 1
			elif argv[i] == '-t' and i < len(argv)-1:
				try:	tmo = string.atoi(argv[i+1])
				except:
					print CPUsage
					return 2
				i = i + 1
			elif argv[i] == '-p' and i < len(argv)-1:
				project = argv[i+1]
				i = i + 1
			elif argv[i] == '-c' and i < len(argv)-1:
				i = i + 1
				cmd = [argv[i]]
				if not cmd or cmd[0] == '-':
					print CPUsage
					return 2
				in_cmd = 1
			else:
				print CPUsage
				return 2
		else:
			cmdopt.append(argv[i])
		i = i + 1
	cfg = ConfigFile(os.environ['FCP_CONFIG'])
	if port == None:	
		port = cfg.getValue('fcp','*','port')
	if not f1 or not f2 or f1[0] == '-' or f2[0] == '-':
		print CPUsage
		return 2
	ic1 = string.find(f1,':')
	ic2 = string.find(f2,':')
	if ic1 > 0 and ic2 < 0:
		# download
		host = string.split(f1,':')[0]
		ruser = ''
		if string.find(host, '@') > 0:
			ruser = string.split(host,'@')[0]
			host = string.split(host,'@')[1]
		rfn = string.split(f1,':')[-1]
		lfn = f2
		
		fcpc = FCPClient(host, ConfigFile(os.environ['FCP_CONFIG']),
				project = project, user = ruser)

		if verbose:
			print 'Request download...'
		sts = 0
		t = time.time()
		while not sts:
			sts, txt, afn = fcpc.requestDownload(rfn, tmo)
			if not sts:
				if txt[:3] == 'ERR':
					print 'Error: %s' % txt[3:]
					return 1
				if tmo >= 0 and time.time() > t + tmo:
					print 'timeout'
					return 3
				else:
					time.sleep(1)
		if afn != rfn:
			if ruser:
				f1 = '%s@%s:%s' % (ruser, host, afn)
			else:
				f1 = '%s:%s' % (host, afn)
		rid = txt
		if verbose:
			cmdstr = string.join(cmd + cmdopt + [f1, f2])
			print 'Request granted.'
			print 'Spawning <%s> ...' % cmdstr
		sts = doTransfer(fcpc, rid, tuple(cmd + cmdopt + [f1, f2]))
		if verbose:
			print 'Transfer status = ', sts
		return sts
	elif ic2 > 0 and ic1 < 0:
		# upload
		host = string.split(f2,':')[0]
		ruser = ''
		if string.find(host, '@') > 0:
			ruser = string.split(host,'@')[0]
			host = string.split(host,'@')[1]
		rfn = string.split(f2,':')[-1]
		lfn = f1

		fcpc = FCPClient(host, ConfigFile(os.environ['FCP_CONFIG']),
				project = project, user = ruser)

		try:	stbuf = os.stat(lfn)
		except os.error, val:
			print 'stat failed: ', val
			return 1
		size = stbuf[stat.ST_SIZE]/(1024*1024) + 1
		if verbose:
			print 'Request upload...'
		sts = 0
		t = time.time()
		while not sts:
			sts, txt, afn = fcpc.requestUpload(rfn, size, tmo)
			if not sts:
				if txt[:3] == 'ERR':
					print 'Error: %s' % txt[3:]
					return 1
				if tmo >= 0 and time.time() > t + tmo:
					print 'timeout'
					return 3
				else:
					time.sleep(1)

		if afn != rfn:
			if ruser:
				f2 = '%s@%s:%s' % (ruser, host, afn)
			else:
				f2 = '%s:%s' % (host, afn)
		rid = txt
		#print cmd + cmdopt + [f1, f2]
		if verbose:
			cmdstr = string.join(cmd + cmdopt + [f1, f2])
			print 'Request granted.'
			print 'Spawning <%s> ...' % cmdstr
		sts = doTransfer(fcpc, rid, tuple(cmd + cmdopt + [f1, f2]))
		if verbose:
			print 'Transfer status = ', sts
		return sts
	else:
		# wrong syntax
		print CPUsage
		return 2
		
def doAbort(argv):
	if len(argv) != 2:
		print Usage
		return 2
	rid = args[1]
	host = args[0]
	fcpc = FCPClient(host, ConfigFile(os.environ['FCP_CONFIG']))
	sts, txt = fcpc.abortRequest(rid)
	if sts:
		return 0
	else:
		print txt
		return 1

def doFind(argv):
	if len(argv) != 2:
		print Usage
		return 2
	host = argv[0]
	vfn = argv[1]
	fcpc = FCPClient(host, ConfigFile(os.environ['FCP_CONFIG']))
	sts, txt = fcpc.translate(vfn)
	print txt
	if not sts: return 1
	return 0
	
def main(argv):
	import getopt
	import pwd
	
	if argv[1] == 'list':
		return doList(argv[2:])
	elif argv[1] == 'cp':
		return doCopy(argv[2:])
	elif argv[1] == 'abort':
		return doAbort(argv[2:])
	elif argv[1] == 'xlate':
		return doFind(argv[2:])
	else:
		print Usage
		return 2

if __name__ == '__main__':
	sys.exit(main(sys.argv))
