# 
# @(#) $Id: fcpd.py,v 1.14 2001/09/10 17:34:08 ivm Exp $
#
# $Log: fcpd.py,v $
# Revision 1.14  2001/09/10 17:34:08  ivm
# Break out of wait loop on fatal error
#
# Revision 1.12  2001/09/10 15:15:02  ivm
# Do import time in fcpc.py
#
# Revision 1.11  2001/03/16 23:16:44  ivm
# Added Install.txt
# Fixed bug in FCPClient.listQueue()
#
# Revision 1.10  2000/12/13 16:47:50  ivm
# Fixed bugs
#
# Revision 1.9  2000/11/21 19:38:30  ivm
# Renamed find to translate
#
# Revision 1.6  2000/11/07 20:48:25  ivm
# Implemented multi-volume placement. Still needs polishing.
#
# Revision 1.5  2000/10/25 22:44:11  ivm
# Tested scheduling and renewal
#
# Revision 1.4  2000/08/25 17:53:15  ivm
# Implemented log file, HELLO message
#
# Revision 1.3  2000/08/14 17:32:30  ivm
# Added fcp.cfg
# Read parameters from fcp.cfg
#
# Revision 1.1  2000/08/14 16:02:01  ivm
# Initial versions
#
#

from TCPServer import TCPServer
import string
from SockStream import SockStream
from txns import *
from psa import *
from Selector import Selector
import time
import pwd
import stat
from socket import *

class	DGC:
	def __init__(self, dg, sock, addr, sel):
		self.DG = dg
		self.Addr = addr
		self.Sock = sock
		self.Str = SockStream(sock,'\n')
		self.Sel = sel
		sel.register(self, rd=self.Sock.fileno())
		self.User = 'unknown'
		self.Project = 'unknown'
		self.CPID = 'unknown'
		self.getHello()
		self.IsWaiting = 0
		self.RID = None

	def log(self, msg):
		self.DG.log('DGC %s %s: %s' % (self.User, self.clientID(), msg))

	def clientID(self):
		try:	node = gethostbyaddr(self.Addr[0])[0]
		except: node = self.Addr[0]
		return '%s@%s' % (self.CPID, node)
						
	def getHello(self):
		# HELLO <user> <pid>
		try:	hello = self.Str.recv(tmo = 10)
		except: 
			self.disconnect()
			return
		if not hello:
			# The other end has disconnected already
			self.disconnect()
			return
		words = string.split(hello)
		if len(words) < 4 or words[0] != 'HELLO':
			self.Str.send('ERR Say HELLO <user> <project> <pid>')
			self.disconnect()
			return
		self.User = words[1]
		self.Project = words[2]
		self.CPID = words[3]
		self.Str.send('OK')
		self.log('connected')
		
	def doRead(self, fd, sel):
		if fd != self.Sock.fileno():
			return
		self.Str.readMore()
		while self.Str != None and self.Str.msgReady():
			msg = self.Str.getMsg()
			msg = string.strip(msg)
			self.log('msg: <%s>' % msg)
			if msg:
				lst = string.split(msg)
				if not lst: continue
				if lst[0] == 'DL':
					self.doDownload(msg)
				elif lst[0] == 'UL':
					self.doUpload(msg)
				elif lst[0] == 'FIN':
					self.doFin(msg)
				elif lst[0] == 'LIST':
					self.doList(msg)
				elif lst[0] == 'RENEW':
					self.doRenew(msg)
				elif lst[0] == 'ABORT':
					self.doAbort(msg)
				elif lst[0] == 'XLATE':
					self.doTranslate(msg)
		
		if self.Str != None and self.Str.eof():
			self.log('EOF')
			self.doEOF()

	def doAbort(self, msg):
		# ABORT <rid> 
		lst = string.split(msg)
		if len(lst) < 2:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		rid = lst[1]
		self.log('Aborting RID=<%s>' % rid)
		self.DG.abortRequest(rid)
		self.Str.send('OK')
		self.disconnect()
				
	def doRenew(self, msg):
		# RENEW <rid> 
		lst = string.split(msg)
		if len(lst) < 2:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		rid = lst[1]
		self.log('Renewing RID=<%s>' % rid)
		self.DG.renewRequest(rid)
		self.Str.send('OK')
		#self.disconnect()
				
	def doDownload(self, msg):
		# DL <path>
		lst = string.split(msg)
		if len(lst) < 2:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		path = lst[1]
		if path[0] != '/':
			try:	path = pwd.getpwnam(self.User)[5] + '/' + path
			except:
				self.Str.send('ERR Can not determine absolute path. Remote user=<%s>' % self.User)
				self.disconnect()
				return
		size = 0		# for now
		try:
			stb = os.stat(path)
			size = stb[stat.ST_SIZE]/(1024*1024)
		except:
			pass	# size is not important
		request = DGRequest(self.DG, self, 'D', path, size)
		self.RID = request.ID
		self.log('Download file <%s> size=%d, RID=<%s>' %
				(path, size, request.ID))
			
		self.DG.enqueueRequest(request)
		self.Str.send('WAIT')
		self.IsWaiting = 1

	def doUpload(self, msg):
		# UL <path> <size>
		lst = string.split(msg)
		if len(lst) < 3:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		path = lst[1]
		size = lst[2]
		try:	size = string.atoi(size)
		except:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		if path[0] != '/':
			try:	path = pwd.getpwnam(self.User)[5] + '/' + path
			except:
				self.Str.send('ERR Can not determine absolute path. Remote user=<%s>' % self.User)
				self.disconnect()
				return
		request = DGRequest(self.DG, self, 'U', path, size)
		self.RID = request.ID
		self.log('Upload file <%s> size=%d, RID=<%s>' %
				(path, size, request.ID))
		self.IsWaiting = 1
		self.DG.enqueueRequest(request)
		if self.IsWaiting:
			# DG.schedule() called by enqueue() may have granted the
			# request already,
			# and GRANTED has been sent, otherwise send WAIT
			self.Str.send('WAIT')

	def doFin(self, msg):
		# FIN <rid>
		lst = string.split(msg)
		if len(lst) < 2:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		rqid = lst[1]
		self.log('Finish RID=%s' % rqid)
		self.DG.finishRequest(rqid)
		self.Str.send('OK')
		self.disconnect()

	def doEOF(self):
		self.DG.abortRequest(self.RID)
		self.RID = None
		self.disconnect()

	def doList(self, msg):
		# LIST
		lst = self.DG.listRequests()
		for r in lst:
			psa = r.PSA
			if psa != None:
				free = psa.freeMB()
				rsrv = psa.reservedByTxns()
			else:
				free = '-'
				rsrv = '-'
			self.Str.send('%s %s %s %s %s %s %d %s %s' % (
				r.ID, r.clientID(), r.user(), r.State, r.Type, r.Path, 
				r.Size, free, rsrv))
		self.Str.send('.')
		#self.disconnect()	

	def doTranslate(self, msg):
		lst = string.split(msg)
		if len(lst) < 2:
			self.Str.send('ERR Protocol error <%s>' % msg)
			self.disconnect()
			return
		vfn = lst[1]
		afn = self.DG.translate(vfn)
		self.log('Translated <%s> to <%s>' % (vfn, afn))
		if not afn:
			self.Str.send('NF')
		else:
			self.Str.send('OK %s' % afn)
		self.disconnect()
		
	def granted(self, afn = ''):
		self.Str.send('GRANTED %s %d %s' % 
			(self.RID, self.DG.renewInterval(), afn))
		self.log('Granted RID=%s Renew=%d File=<%s>' % (self.RID, self.DG.renewInterval(), afn))
		self.IsWaiting = 0
		#self.disconnect()

	def aborted(self):
		self.log('Aborted RID=%s' % self.RID)
		self.IsWaiting = 0
		if self.Str:
			self.Str.send('ABORTED')
			self.disconnect()

	def disconnect(self):
		if self.Str:
			self.log('disconnecting')
			self.Sel.unregister(rd=self.Sock.fileno())
			self.Sock.close()
			self.Str = None
			self.Sock = None

class	DGRequest:
	_NextID = 1
	
	def __init__(self, dg, clnt, type, path, size):
		self.ID = dg.nextRID()
		#DGRequest._NextID = (DGRequest._NextID % 9999) + 1
		self.Path = path
		self.DG = dg
		self.Client = clnt
		self.Size = size
		self.Type = type
		self.State = 'W'
		self.AbortAfter = None
		self.Txn = None
		self.Project = clnt.Project
		self.AFN = None
		self.PSA = None
		self.SubTime = time.time()
		self.StartTime = None
		self.EndTime = None
		relpath, plst = self.DG.logicalPSA(path)
		if not plst:
			self.AFN = path
			self.PSA = self.DG.findPSA(path)
				
	def log(self, msg):
		self.DG.log('RID=%s: %s' % (self.ID, msg))
			
	def canStart(self):
		if self.Type == 'U':
			# check sizes
			afn = self.DG.translate(self.Path)
			psa = self.DG.findPSA(afn)			
			#print 'PSA for <%s> is' % afn, psa
			if self.Size > psa.free(): return 0
		return self.DG.canStartTxn(self.Type, self.Path)

	def start(self):
		#print 'Request #%d, start: txn=', txn
		self.AFN = self.DG.translate(self.Path)
		self.PSA = self.DG.findPSA(self.AFN)
		if self.Type == 'D':
			self.Txn = DLTxn(self.Size, self.PSA, self.Path)
		elif self.Type == 'U':
			self.Txn = ULTxn(self.Size, self.PSA, self.Path)
		self.StartTime = time.time()
		self.Txn.notify(self.DG)
		self.State = 'R'
		self.AbortAfter = time.time() + self.DG.TxnTimeout
		return self.AFN

	def end(self, state):
		self.EndTime = time.time()
		self.log('ACCT %s Sts:%s Size:%d Sub:%s Start:%s (%s) End:%s (%s)' %
			(self.Type, state, self.Size, self.cvatime(self.SubTime),
				self.cvatime(self.StartTime),
				self.cvdtime(self.SubTime, self.StartTime),
				self.cvatime(self.EndTime),
				self.cvdtime(self.StartTime, self.EndTime)))

	def cvatime(self, t):
		if type(t) != type(1) and type(t) != type(1.0):
			return '?'
		t = float(t)
		y,m,d,hh,mm,ss = time.localtime(t)[:6]
		return '%02d/%02d %02d:%02d' % (m,d,hh,mm)
						
	def cvdtime(self, t1, t2):
		if type(t1) != type(1) and type(t1) != type(1.0):
			return '?'
		if type(t2) != type(1) and type(t2) != type(1.0):
			return '?'
		dt = int(abs(t2-t1))
		s = dt % 60
		m = (dt/60) % 60
		h = dt/3600
		if h:
			return '%dh%d' % (h,m)
		else:
			return '%dm%d' % (m,s)
					
	def inProgress(self):
		return self.State == 'R'

	def canAbort(self):
		return self.AbortAfter != None and time.time() > self.AbortAfter
		
	def renew(self):
		if self.State == 'R':
			t = time.time() + self.DG.renewExtent()
			if self.AbortAfter < t:
				self.AbortAfter = t

	def clientID(self):
		return self.Client.clientID()

	def user(self):
		return self.Client.User
		
class	DGQueue:
	def __init__(self, cfg):
		self.ProjQueues = {}
		self.PrioList = []		# list of high priority projects
		self.OtherProjs = []	# list of low priority projects
		self.ProjPrio = cfg.getValueDict('fcp','*','priorities',defValue=-1)
		if self.ProjPrio == None:
			self.ProjPrio = {}

	def projPrio(self, proj):
		if self.ProjPrio.has_key(proj):
			return self.ProjPrio[proj]
		else:
			# lowest priority
			return None
		
	def addRequest(self, rq):
		proj = rq.Project
		if not self.ProjQueues.has_key(proj):
			self.ProjQueues[proj] = []
			p = self.projPrio(proj)
			if p == None:
				self.OtherProjs.append(proj)
			else:
				self.PrioList.append(proj)
		self.ProjQueues[proj].append(rq)
		
	def removeRequest(self, rid):
		for proj, pq in self.ProjQueues.items():
			for i in range(len(pq)):
				if pq[i].ID == rid:
					self.ProjQueues[proj] = pq[:i] + pq[i+1:]
					break

	def grantProjectRequest(self, proj):
		# move project to the end of list according to
		# its priority 
		p = self.projPrio(proj)
		if p == None:
			# round-robin low priority projects
			self.OtherProjs.remove(proj)
			self.OtherProjs.append(proj)
		else:
			if p < 0:
				# move project to the end
				self.PrioList.remove(proj)
				self.PrioList.append(proj)
			elif p == 0:
				# do not move at all
				return
			else:
				# move as specified
				i = self.PrioList.index(proj)
				j = i + p + 1
				self.PrioList = self.PrioList[:i] + \
					self.PrioList[i+1:j] + [proj] + \
					self.PrioList[j:]

	def schedule(self):
		# go through list of requests sorted by position in
		# PrioList and then OtherProjs
		for proj in self.PrioList + self.OtherProjs:
			for r in self.ProjQueues[proj]:
				if not r.inProgress() and r.canStart():
					# grant this request
					self.grantProjectRequest(proj)
					return r
		return None

	def list(self):
		lst = []
		for pq in self.ProjQueues.values():
			lst = lst + pq
		return lst

class	DiskGuard(TCPServer, HasTxns):
	def __init__(self, cfg, sel):
		self.Port = cfg.getValue('fcp','*','port')
		self.MaxTxns = cfg.getValue('fcp','*','max_xfers')
		self.TxnTimeout = cfg.getValue('fcp','*','timeout') * 60
		self.LogFileName = cfg.getValue('fcp','*','log_file','/tmp/fcp_log')
		# read configuration
		self.PSAs = {}				# name -> PSA
		self.Queue = DGQueue(cfg) 		# request queue
		self.Requests = {}			# rqID -> request
		self.LVDict = {}
		for lvn in cfg.names('logical_volumes','*'):
			self.LVDict[lvn] = cfg.getValueList('logical_volumes','*',lvn)
		self.VServers = {}
		self.VSCount = {}
		for vsn in cfg.names('virtual_servers','*'):
			self.VServers[vsn] = cfg.getValue('virtual_servers','*',vsn)
			self.VSCount[vsn] = 0
		self.LastScdTime = 0
		self.ScdInterval = 60	# seconds
		self.NextRID = 1
		self.MaxRID = 99999
		TCPServer.__init__(self, self.Port, sel)
		HasTxns.__init__(self)

	def nextRID(self):
		done = 0
		rid = None
		while not done:
			rid = '%d' % self.NextRID
			self.NextRID = (self.NextRID % self.MaxRID) + 1
			done = not self.Requests.has_key(rid)
		return rid

	def renewInterval(self):
		return self.TxnTimeout/5
		
	def renewExtent(self):
		return self.renewInterval()*3
		
	def log(self, msg):
		tup = time.localtime(time.time())
		y, mo, d, h, mi, s = tup[:6]
		fn = '%s.%04d-%02d-%02d.log' % (self.LogFileName, y, mo, d)
		stamp = '%04d/%02d/%02d %02d:%02d:%02d' % (y, mo, d, h, mi, s)
		try:	f = open(fn, 'a')
		except: return
		f.write('%s %s\n' % (stamp, msg))
		f.close()
		#print stamp, msg

	def createClientInterface(self, sock, addr, sel):
		DGC(self, sock, addr, sel)

	def enqueueRequest(self, rq):
		self.Queue.addRequest(rq)
		self.Requests[rq.ID] = rq
		self.schedule()
		
	def finishRequest(self, rid):
		self.Queue.removeRequest(rid)
		try:	rq = self.Requests[rid]
		except KeyError:
			return
		rq.end('F')
		txn = rq.Txn
		if txn != None:
			txn.commit()
		del self.Requests[rid]
		self.schedule()
		
	def abortRequest(self, rid):
		self.Queue.removeRequest(rid)
		try:	rq = self.Requests[rid]
		except KeyError:
			return
		rq.end('A')
		if rq.State == 'R':
			txn = rq.Txn
			txn.rollback()
		else:
			if rq.Client != None:
				rq.Client.aborted()
		del self.Requests[rid]
		self.schedule()
		
	def cancelRequest(self, rid):
		# Cancel pending request. There is no transaction yet.
		self.Queue.removeRequest(rid)
		try:	rq = self.Requests[rid]
		except KeyError:
			pass
		rq.end('C')
		del self.Requests[rid]
		self.schedule()

	def renewRequest(self, rid):
		try:	self.Requests[rid].renew()
		except KeyError:
			pass
	
	def run(self):
		if self.cleanUp() or \
				time.time() > self.LastScdTime + self.ScdInterval:
			self.schedule()

	def cleanUp(self):
		sts = 0
		for rid, r in self.Requests.items():
			if r.State == 'R' and r.AbortAfter < time.time():
				self.abortRequest(rid)
				sts = 1
		return sts

	def schedule(self):
		tryAgain = 1
		self.LastScdTime = time.time()
		while tryAgain:
			tryAgain = 0
			if self.canStartTxn():
				rq = self.Queue.schedule()
				if rq:
					self.log('SCD: start RID=%s' % rq.ID)
					afn = rq.start()
					rq.Client.granted(afn)
				tryAgain = not (not rq)

	def canStartTxn(self, type = 'D', path = None):
		#print '%d txns: ' % self.txnCount(), self.Txnd
		if self.txnCount() >= self.MaxTxns:	return 0
		if path:
			vsns = self.findVirtualServers(path)
			for vsn in vsns:
				if self.VSCount[vsn] >= self.VServers[vsn]:	
					return 0
		return 1
		
	def findPSA(self, path):
		devno  = None
		try:	devno = os.stat(path)[2]
		except:
			# file not found. Try directory
			while path and devno == None:
				inx = string.rfind(path, '/')
				if inx < 0:
					break
				path = path[:inx]
				if not path:	path = '/'
				try:	
					devno = os.stat(path)[2]
				except: pass
			if not devno:
				return None
		# find topmost directory with the same devno
		topmost = path
		thedev = devno
		while devno == thedev and len(path) > 1:
			topmost = path
			#print 'path = <%s>' % path
			# truncate last path component
			inx = string.rfind(path, '/')
			if inx >= 0:
				path = path[:inx]
			try:	devno = os.stat(path)[2]
			except: devno = -1
		if not self.PSAs.has_key(topmost):
			self.PSAs[topmost] = PSA(topmost, topmost)
		return self.PSAs[topmost]

	def listRequests(self):
		return self.Queue.list()

	def canonicLogicalPath(self, path):
		# does not handle '.' and '..' !!
		while path:
			i = string.find(path, '//')
			if i >= 0:
				path = path[:i] + '/' + path[i+2:]
			else:
				break
		# it is always absolute
		if path[:1] != '/':
			path = '/'+path
		return path

	def findVirtualServers(self, path):
		path = self.canonicLogicalPath(path)
		items = filter(lambda x: x and 1, string.split(path, '/'))
		paths = []
		p = ''
		for i in items:
			p = p + '/' + i
			if self.VServers.has_key(p):
				paths.append(p)
		return paths

	def openTxn(self, tid, txn):
		path = txn.Path
		for p in self.findVirtualServers(path):
			self.VSCount[p] += 1

	def commitTxn(self, tid, txn):
		path = txn.Path
                for p in self.findVirtualServers(path):
                        self.VSCount[p] -= 1

	rollbackTxn = commitTxn

	def logicalPSA(self, lfn):
		# strip filename
		path = lfn
		fn = ''
		if path[-1] != '/':
			lst = string.split(path,'/')
			path = string.join(lst[:-1],'/')
			fn = lst[-1]
		#print 'path = <%s>, fn = <%s>' % (path, fn)
		
		pvlst = None
		relpath = lfn
		#print self.LVDict
		for lvn, pvl in self.LVDict.items():
			if '/' + lvn == path[:len(lvn)+1]:
				#print 'match <%s>: %s' % (lvn, pvl)
				pvlst = pvl
				relpath = path[len(lvn)+2:] + '/' + fn
				if relpath[0] == '/':	relpath = relpath[1:]
				#print 'relpath=<%s>' % relpath
				break
		return relpath, pvlst
	
	def translate(self, lfn):
		# returns actual file name
		relpath, pvlst = self.logicalPSA(lfn)
		#print relpath, pvlst
		if not pvlst:	return	lfn
		# try to find the file
		for pv in pvlst:
			afn = pv + '/' + relpath
			try:	os.stat(afn)			
			except: continue
			else:	return afn
		#print 'file not found'	
		# file not found, find the biggest PSA
		thePSA = None
		maxsize = None
		afn = None
		for pv in pvlst:
			psa = self.findPSA(pv)
			f = psa.free()
			#print 'PSA <%s>, free %d' % (psa.Root, f)
			if maxsize == None or f > maxsize:
				thePSA = psa
				maxsize = f
				afn = pv + '/' + relpath
		#print 'returning afn=<%s>' % afn
		return afn

if __name__ == '__main__':
	from config import *
	cfg = ConfigFile(os.environ['FCP_CONFIG'])
	sel = Selector()
	dg = DiskGuard(cfg, sel)
	while 1:
		sel.select(10)
		dg.run()
