"""This python script is command line xmldb client """

__author__="Parag Mhashilkar <parag@fnal.gov>"
__date__= "April 22 2003"
__version__="0.1"

# $source: JimConfigManagerCommand.py,v$
 
################################################################################
# Project   : SAM
# Package : jim_config
#
# This work is part of a development project, called JIM, which consists of a
# number of coordinated packages each named jim_xxxx .
#
# Notice of authorship, copyright status,  and terms and conditions, should
# the software eventually become available for use outside Fermilab, can be
# found in the README and LICENCE files in the top level directory of the main
# sam package.
################################################################################


import sys, os
from JimConfigManager import JimConfigManager


#####################################################################################
# Class: JimConfigManagerCommand
#####################################################################################


class JimConfigManagerCommand:
   """ This class provides API for command-line JimConfigManager"""

   __API_Name_="JimConfigManagerCommand"
   __API_Version_="0.2"

   class JCMCommandException(Exception):
      def __init__(self, value):
         self.value = value

      def __str__(self):
         result = {'jim_config_manager_cmd.Fault':"'<"+self.value +"/>'"}
         return result ['jim_config_manager_cmd.Fault']

   # class JCMCommandException

   def __init__(self):
      """ This class initialize variables"""

      self.mode = 'DBFS'

      #Helper
      self.helper = {
         #action : [Action String, No of arguments, [valid options], Usage]
         'gca' :["getConfigAttr", ["-n","-p","-a","-m"], \
                 "jim_config_manager_cmd gca -n <package_name> -p xpath -a attribute -m mode"],
         'gcs' :["getConfigString",["-n","-m"], \
                 "jim_config_manager_cmd gcs -n <package_name> -m mode"],
         'gga' :["getGlobalConfigAttr",["-p","-a","-m"],\
                 "jim_config_manager_cmd gga -p xpath -a attribute -m mode"],
         'ggs' :["getGlobalConfigString",["-m"], \
                 "jim_config_manager_cmd ggs -m mode"],
         'gsa' :["getSiteConfigAttr",["-p","-a","-m"],\
                 "jim_config_manager_cmd gsa -p xpath -a attribute -m mode"],
         'gss' :["getSiteConfigString",["-m"],\
                 "jim_config_manager_cmd gss -m mode"],
         'scf' :["storeConfigFile",["-n","-f","-m"], \
                 "jim_config_manager_cmd scf -n <package_name> -f <filename> -m mode"],
         'scs' :["storeConfigString",["-s","-m"], \
                 "jim_config_manager_cmd scs -s <config_string> -m mode"],
         'sgf' :["storeGlobalConfigFile",["-f","-m"], \
                 "jim_config_manager_cmd sgf -f <filename> -m mode"],
         'sgs' :["storeGlobalConfigString",["-s","-m"], \
                 "jim_config_manager_cmd sgs -s <config_string> -m mode"],
         'ssf' :["storeSiteConfigFile",["-f","-m"], \
                 "jim_config_manager_cmd ssf -f <filename> -m mode"],
         'sss' :["storeSiteConfigString",["-s","-m"], \
                 "jim_config_manager_cmd sss -s <config_string> -m mode"],
      }

      # Client commands
      self.clicommands = {
         # Option : Method Invocation
         '--help' :self.PrintHelp,
         '--version' :self.PrintApiVersion
      }
   # __init__
   
   
   def PrintDefault(self):
      """ Default print script"""

      print "Options:"
      print "    --help [command]"
      print "    --version"
      self.PrintHelp()
   # PrintDefault


   def PrintHelp(self,element=None):
      """Print Help for commands"""

      if (element == None):
         print
         print "Usage: jim_config_manager_cmd --help [command]"
         print "commmand         operations"
         for name,value in self.helper.items():
            print " %s   ----->   %s" %(name,value[0])
      else:
         try:
            print
            print "Usage: %s  \n   %s" %(element,self.helper[element][2])
            print "\nOptions:"
            for value in self.helper[element][1]:
               if(value == "-n"):
                  print "%s   Package name" %value
               elif(value == "-f"):
                  print "%s   File name " %value
               elif(value == "-s"):
                  print "%s   XML string " %value
               elif(value == "-p"):
                  print "%s   XPath to the key" %value
               elif(value == "-a"):
                  print "%s   Attribute name" %value
               elif(value == "-m"):
                  print "%s   Mode (DBFS or FS)" %value
         except KeyError, e:
            print "Command %s invalid" %element
            self.PrintHelp()
      print "Environment variables:"
      print "  JIM_CONFIG_MACHINE: looks at the config of a machine != `hostname`"
      print "  JIM_CONFIG_FS_CONFIG_FILE: the config file used in FS mode"
   # PrintHelp


   def PrintApiVersion(self):
      """ Print the API Version and API Name"""

      print "%s API Version = %s" %(self.__API_Name_,self.__API_Version_);
   # PrintApiVersion


   def setMode(self,mode):
      """See if mode is provided by user"""

      self.mode=mode;
   # setMode
	

   def CheckOptions(self,elements):
      """If the options are valid"""

      self.dict={}
      for i in (range(2,len(elements[2:])+2,2)):
         try:
           self.dict[elements[i]] = elements[i+1]
         except:
           pass
      

      for value in self.helper[elements[1]][1]:
         if(self.dict.has_key(value) == 0):
            if (value == "-m"):
               self.setMode('DBFS')
            else:
               sys.stderr.write("Option %s not found\n " %(value))
               return 1;
         else:
            if (value == "-m"):
               # Set it to the user specific mode
               self.setMode(elements[i+1])
            
      return 0;
   # CheckOptions


   def InvokeApi(self,command):
      """If the options are valid"""

      list = []
      for element in self.helper[command][1]:
         if (element != "-m") and (element != "-n"):
            list.append(self.dict[element])

      try:
         jcm = JimConfigManager(self.dict['-n'])
      except KeyError:
         # It is called for site or global config
         # package name has no real meaning. Default it to jim_config
         jcm = JimConfigManager('jim_config')
         
      jcm.setMode(self.mode)

      # Commands
      commands = {
         #action : Method Invocation 
         'gca' :[jcm.getConfigAttr],
         'gcs' :[jcm.getConfigString],
         'gga' :[jcm.getGlobalConfigAttr],
         'ggs' :[jcm.getGlobalConfigString],
         'gsa' :[jcm.getSiteConfigAttr],
         'gss' :[jcm.getSiteConfigString],
         'scf' :[jcm.storeConfigFile],
         'scs' :[jcm.storeConfigString],
         'sgf' :[jcm.storeGlobalConfigFile],
         'sgs' :[jcm.storeGlobalConfigString],
         'ssf' :[jcm.storeSiteConfigFile],
         'sss' :[jcm.storeSiteConfigString],
      }
   
      return (commands[command][0])(*list)
   # InvokeApi
