"""
This python library makes managing the product configuration easy
"""

__author__="Parag Mhashilkar <parag@fnal.gov>"
__date__= "May 05 2003"
__version__="1.0"

# $source: ProductConfigManager.py,v$

################################################################################
# Project   : SAM
# Package : jim_config
#
# This work is part of a development project, called JIM, which consists of a
# number of coordinated packages each named jim_xxxx .
#
# Notice of authorship, copyright status,  and terms and conditions, should
# the software eventually become available for use outside Fermilab, can be
# found in the README and LICENCE files in the top level directory of the main
# sam package.
################################################################################

import os, string, sys, re
import xml_common_lib
from BaseDBConfigManager import BaseDBConfigManager
from BaseFSConfigManager import BaseFSConfigManager

#####################################################################################
# Class: ProductDBConfigManager
#####################################################################################
 

class ProductDBConfigManager(BaseDBConfigManager):
   """This utility class manages the configuration storing and retrival from Db"""

   def __init__(self, name, version=None, qual=None, machine=None):
      """
         Contructor to create a ProductDBConfigManager

         name - Name of the package
      """

      try:
        BaseDBConfigManager.__init__(self, name)
        # Name of the product
        self.name = name
        if version:
           self.version = version
        else:
           self.version = self.getPrdVersion()
        if qual:
           self.qualifiers = qual
        else:
           self.qualifiers = self.getPrdQualifiers()
        if machine:
           self.machine = machine
        else:
           self.machine = self.getPrdMachine()

        self.context = self.myContext()
        self.configDoc = self.getPrdConfigDocName()
        self.printConfigDetails()
      except:
         import traceback
         if self.getDebugMode(): traceback.print_exc()
         raise self.ConfigManagerError('error XML DB config error')

   # __init__

   
   def printConfigDetails(self):
      self.printDebug('-------------------------------------------------------')
      self.printDebug('Product Name          : ' + self.name)
      if self.version:
         self.printDebug('Product Version       : ' + self.version)
      else:
         self.printDebug('Product Version       : ')
      if self.qualifiers:
         self.printDebug('Product Qualifiers    : ' + self.qualifiers)
      else:
         self.printDebug('Product Qualifiers    : ')
      if self.machine:
         self.printDebug('Product Machine       : ' + self.machine)
      else:
         self.printDebug('Product Machine       : ')
      self.printDebug('Generated context     : ' + self.context)
      self.printDebug('Name of configuration : ' + self.configDoc)
      self.printDebug('-------------------------------------------------------')
   # printConfigDetails


   def myContext(self):
      context = self.getPrdCollection()
      for prd in self.getHostDependentPrdList():
         if prd == self.name:
            context = context + '/' + self.machine
            break
 
      context = re.compile('//+').sub('/',context)

      return context
   # myContext


#####################################################################################
# Class: ProductFSConfigManager
#####################################################################################
 

class ProductFSConfigManager(BaseFSConfigManager):
   """This utility class manages the configuration storing and retrival from Db"""

   def __init__(self, name, version=None, qual=None, machine=None):
      """
         Contructor to create a ProductFSConfigManager

         name - Name of the package
      """

      BaseFSConfigManager.__init__(self, name)

      self.name = name
      if version:
         self.version = version
      else:
         self.version = self.getPrdVersion()
      if qual:
         self.qualifiers = qual
      else:
         self.qualifiers = self.getPrdQualifiers()
      if machine:
         self.machine = machine
      else:
         self.machine = self.getPrdMachine()

      self.path = self.getPrdPath()

      # Name of the configuration document depends on hostname in FS
      self.configDoc = self.myConfigDocName()
      self.printConfigDetails()
   # __init__


   def printConfigDetails(self):
      self.printDebug('-------------------------------------------------------')
      self.printDebug('Product Name          : ' + self.name)
      if self.version:
         self.printDebug('Product Version       : ' + self.version)
      else:
         self.printDebug('Product Version       : ')
      if self.qualifiers:
         self.printDebug('Product Qualifiers    : ' + self.qualifiers)
      else:
         self.printDebug('Product Qualifiers    : ')
      if self.machine:
         self.printDebug('Product Machine       : ' + self.machine)
      else:
         self.printDebug('Product Machine       : ')
      self.printDebug('Path to configuration : ' + self.path)
      self.printDebug('Name of configuration : ' + self.configDoc)
      self.printDebug('-------------------------------------------------------')
   # printConfigDetails


   def myConfigDocName(self):
      return  self.getPrdConfigDocName()
   # myConfigDocName


# Class ProductFSConfigManager
