
#!/bin/sh

#
# Step 1: stash the proxy in a place where the gridftp daemon cannot erase it
# (we outsmart it!)
#
prog=`basename $0`
die () { echo "$prog: $@" >&2; exit 1; }


###DEBUG...
#env
###...DEBUG


user_proxy=${1:-${X509_USER_PROXY}}

## Test all the bloody assumptions here please!
[ -n "${user_proxy}" ] || \
  die "$0: Specify user proxy via \$1 or \$X509_USER_PROXY"
[ -n "$GLOBUS_LOCATION" ] || die "GLOBUS_LOCATION not set"
[ -x $GLOBUS_LOCATION/sbin/in.ftpd ] || die "Gridftp server not present"

#
# Another Assumption: Trusted CA's are available here
#

local_gridftpd_dir=${JIM_LOCAL_GRIDFTP_DIR:-`pwd`/gridftpd}
#[ ! -x $local_gridftpd_dir ] || die "Local gridftpd directory exists already at $local_gridftpd_dir"
proxy_dir=${local_gridftpd_dir}/non_erasable
mkdir -p $proxy_dir || die "Cannot make subdirectories"
proxy_dir=`cd $proxy_dir; pwd`

# Copy the proxy and generate a gridmapfile
cp $user_proxy ${proxy_dir} || die "Bad proxy '$user_proxy'"
user_proxy=$proxy_dir/`basename $user_proxy`
gridmap=${proxy_dir}/grid-mapfile

#server_subject=`X509_USER_CERT=${user_proxy} grid-cert-info -subject`
#subject=`grid-proxy-info -subject -file ${user_proxy}`
subject=`grid-cert-info -subject -file ${user_proxy}`
#Manually fixes openssl versions that do not know about object USERID
subject=`echo $subject | sed 's/0\.9\.2342\.19200300\.100\.1\.1/USERID/g'`

[ -n "$subject" ] || die "Cannot get proxy's subject"
echo "\"$subject\" `whoami`" > ${gridmap} || die "Cannot create gridmapfile"


# Make th directory write protected to prevent gridftp from deleting it
chmod u-w $proxy_dir || die "Cannot hide proxy"

#
# Step 2 pick a good guess for an unused port and try it
# repeat a few times
port=$$
num_tries=10
#sid=`ps -h -p $$ -o sid`
success=""


if [ -n "$SAM_GSI_CONFIG_DIR" ]; then
  echo "$prog: will read X509_CERT_DIR from sam_gsi_config"
  export X509_CERT_DIR=`sam_gsi_read_config SAM_GSI_JIM_GRIDFTP_CERT_DIR`
  echo "$prog: X509_CERT_DIR=${X509_CERT_DIR}"
else
  echo "$prog: WARNING using the default gridmap file"
fi

while [ ${num_tries} -gt 0 ]; do
 port=$((1024 + $(($port % 64512 ))))


# X509_USER_PROXY=${user_proxy} X509_CERT_DIR=${proxy_dir}/certificates 
 #Oct 3: if we don't send stdout and stderr of the server to /dev/null
 #       popen() from python waits forever for the dup'ed file descriptors
 #       to be closed

 X509_USER_PROXY=${user_proxy}  \
   GRIDMAP=${gridmap}\
   $GLOBUS_LOCATION/sbin/in.ftpd -d -s -A -Q -W -p $port 2>&1 > /dev/null &
 sleep 3
 
 #test if started OK
 pid=`jim_dynamic_gridftpd_tool get_pids`


 #pid=`ps -o pid,sid,args | egrep "$sid..*ftpd" | \
 #     fgrep -v grep | sed "s/^ *//g" | cut -f1 -d' '`

 if [ -n "$pid" ]; then success="true"; break; fi                    
 echo "$0: Port $port didn't work out" >&2
 num_tries=$(($num_tries-1))

done

if [ -n $success ]; then echo "$prog: Started with pid $pid on port $port">&2; fi

#The subject of the server is determined from cert info not proxy info!!!
subject_file=`jim_dynamic_gridftpd_tool get_subject_file`
#echo $daemonSubject > $subject_file
server_subject=`grid-cert-info -subject -file ${user_proxy}`
echo $server_subject > $subject_file

if [ -x $GLOBUS_LOCATION/bin/globus-url-copy ]; then
  tmp_file=/tmp/gridftp_test.$$
  echo $pid > $tmp_file
  target_pid_file=${local_gridftpd_dir}/pid
  rm -f $target_pid_file
  #Compatibility with globus 2.0: option -nodcau
  cmd="X509_USER_PROXY=${user_proxy} \
       $GLOBUS_LOCATION/bin/globus-url-copy \
        -nodcau \
 	-s \"$server_subject\" file://${tmp_file} \
	 gsiftp://`hostname -f`:${port}${target_pid_file}"

  echo "X509_CERT_DIR=$X509_CERT_DIR"
  echo "$prog: Testing our little server with command $cmd"
  eval $cmd
  rm -f $tmp_file
  echo "$prog: if the test was successful, the $target_pid_file has been created"
  ls -laF $target_pid_file || { kill -9 $pid; die "gridFTP test failed"; }
else
  echo "$prog: WARNING, no gridgtp client, cannot test the server" >& 2
fi

