import os
import sys
import time
import random
import traceback
import string
import commands
import re

#from SamCorba.SamServerProxyManager import getSamServerProxyManager
from Sam import sam
from samProject import dhGetLite, Printers
from SamStruct.DataFilePhysicalAttributesList import DataFilePhysicalAttributesList
from SamStruct.DataFilePhysicalAttributes import DataFilePhysicalAttributes
import SAMDbServer
import MetadataHelper
from SamFile.SamDataFile import SamDataFile


class FetchedData:
  def __init__(self,files,path):
    self._files = files
    self._path = path
  
  def release(self,what):
    pass

  def getData(self): 
    return self._files

  def getPath(self):
    return self._path


class LocalyStoredFetchedData(FetchedData):

  def __init__(self,files,path):
    FetchedData.__init__(self,files,path) 

  def release(self,what = None):
    if ( not what  ):
      os.system("""cd %s; cmd="rm %s"; echo Doing,$cmd; $cmd""" % ( self._path, string.join(self._files," ")))
    else:
       os.system("""cd %s; cmd="rm %s"; echo Doing,$cmd; $cmd""" % ( self._path,what))



def getDsFileIdList(mergeDsName):
      try :

         fileNameList = getFileListByName(mergeDsName,"and availability_status all")
	 # fileNameList is of type DataFilePhysicalAttributesList containing strangely enough
	 # a list of SamStruct.DataFilePhysicalAttributes data structures
         fileIdList = []
         for fileItem in fileNameList:
            print fileItem.get("fileName")
            md = MetadataHelper._samMDCache.get(fileItem.get("fileName"))
            fileIdList.append(md.get("fileId"))
         # end for
         return fileIdList
      except Exception,ex:
	 print "error in Fetcher.getDsFileIdList"
	 print ex
         return []



def getFileListByName(defname,furtherConstraint=""):
     return translateConstraints("__SET__ %s %s" % (defname,furtherConstraint))


def translateConstraints(dim):
      try :
	  result = sam.translateConstraints(dimensions=dim, v=1)
      except Exception,ex :
          print ex
          raise ex

      if len(result) > 0: return result
      return []



def getSnapFileIDList(snapId):
    print "using %s to get FileIDList" % (snapId)
    dim = "DATASET_ID %s" % (snapId)
    fileIDList = []
    results = translateConstraints(dim)
    for res in results:
        fileIDList.append(res.get("fileId"))
    return fileIDList 



def getFileIdList(mergeDsId,mergeDsName):
  
    fileIdList = []
 
    if (mergeDsId == None) and (mergeDsName == None) :
         raise Exception("No way to fetch files to be merged")
    elif ( mergeDsId != None ):
         fileIdList = getSnapFileIDList(mergeDsId)
    else:
         fileIdList = getDsFileIdList(mergeDsName)

    return fileIdList



class FetchStream:

  def __init__(self,inDir,fileIdList,commonargs):
    self._inDir = inDir
    self._print = Printers("FetchStream")
    self._commonargs = commonargs
    self._fileIdList = fileIdList
    self._print.info("File stream: "+str(self._fileIdList))
    # start project / connect to project


  def getFileIDs(self):
    return self._fileIdList




  def fetch(self,chunkSize):

      if ( len(self._fileIdList) == 0 ):
         return None

      cwd = os.getcwd()
      self._print.info("Staging files")
      try :
        os.system("[ -d %s ] || mkdir -p %s" %(self._inDir,self._inDir))
        os.chdir(self._inDir)
      except Exception,ex :
        self._print.error("Could not create/change directory %s" % inDir)
        raise ex

      args = self._commonargs
      
      # stage 'em in chunks
      if ( len(self._fileIdList) < chunkSize): 
         args['fileList'] = self._fileIdList
         self._fileIdList = [] 
      else:
         args['fileList'] =  self._fileIdList[:chunkSize]
         self._fileIdList = self._fileIdList[chunkSize:]

      #print "staging",args['fileList']
      fileNameList = []
      results      = []
      try :
         fileIdList,results = self.getFiles(args)
      except Exception,ex :
         print "Error in getFiles, Files were not retreived successfully"
         import traceback
         traceback.print_exc(file=sys.stderr)
         raise ex
      
      try :
         if not fileIdList or not len(fileIdList):
            raise Exception("dhGetLite returned blank file list")

	 self._print.info("got files successfully")

         if not results or not len(results):
            self._print.info("Mapping file ids to file names")

            for fid in fileIdList:
               fileName = MetadataHelper._metaAttributes.getFileName(fid)
               fileNameList.append(fileName)
         else :            # use results returned by dhgetlite 
            for res in results :
               if not(res['truncated']):
                  fileNameList.append(res['fileName'])
               else :      # ignore truncated files to avoid a file having 2 merged tmbs as children
                  self._print.warning("Ignoring truncated file %s" % res['fileName'])
                  os.unlink(res['fileName'])

         self._print.info("There are %d files to be merged" % len(fileNameList))
         os.chdir(cwd)

         return LocalyStoredFetchedData(fileNameList,self._inDir)
 
      except Exception,ex :
         import traceback
         traceback.print_exc(file=sys.stderr)
         self._print.error(ex)
         self._print.error("Failed to stage files to be merged")
         raise ex



  def mapFileIdToName(self,fileId):
      try :
         md = sam.getMetadata(fileId=fileId)
         return md.get("fileName")
      except Exception,ex:
         self._print.error("Could not get file name for fileid %d" % fileId)
         raise ex
   ## ==== end of mapFileIdToName



  def getFiles(self,args):
      try :
	 #args['d'] = 1
         dhglite = dhGetLite()
         return dhglite.getImplementation()(args,[])
      except Exception,ex:
         print ex
         self._print.error("Failed to fetch dataset")
         raise ex 
  ## ==== end of getFiles


  def getFileIDList(self, snapId):
      return getSnapFileIDList(snapId)


  def getNumFilesForSize(self,totSize):
      return MetadataHelper.getNumFilesFromListForSize(self._fileIdList,totSize)

#d = FetchStream(".",193706,None,None,{"station":"samgfarm"}).fetch(5)

#d.getData()

#os.system("cd %s; ls -l %s" %( d.getPath(), string.join(d.getData())))
#d.release()
#os.system("cd %s; ls -l %s" %( d.getPath(), string.join(d.getData())))

