#!/usr/bin/env python
################################################################################
# Project   : SAM
# Package : 
# $Id: MCMergeImplementation.py,v 1.16 2007/05/21 19:34:14 abaranov Exp $
#
# This work is part of a development project, called SAM, which consists of a
# number of coordinated packages each named sam_xxxx .
#
# Notice of authorship, copyright status,  and terms and conditions, should
# the software eventually become available for use outside Fermilab, can be
# found in the README and LICENCE files in the top level directory of the main
# sam package.
#
# This file is used to run a project over a setup file configured
# by the user.  Please see project_setup.py for more information.
#
################################################################################

import os
import sys
import time
import random
import traceback
import string
import commands
import re

from Sam import sam
import samProject
  
import Fetcher
import MergeFlow
import MetadataHelper

class SamMDCache:
    def __init__(self):
        self._md = {}
        self._print = samProject.Printers("SamMDCache")
    def get(self, filename):
        if self._md.has_key(filename):
            #cache hit
            return self._md[filename]
        try :
	   self._md[filename] = sam.getMetadata(filename=filename) 
        except Exception,ex :
           self._print.error("Failed to get metadata for file %s" % filename)
	   raise ex
	
        return self._md[filename]
## ============== end of class SamMDCache ============== ##

_samMDCache = SamMDCache()


class MCMergeImplementation :
   def __init__ (self,argDict):
      # the only global SamMDCache object
      self._samMDCache = SamMDCache()
      self._print = samProject.Printers("merge")      

      # till we get findFileByEvent sorted out in DhGet (make it like in dhGetLite)
      self._withproject = argDict.get('withproject')
      if not self._withproject == None:
         raise Exception("It is not safe to merge files using --with-project\
 option because of some unimplemented functionalities")
 
      self._commonargs = {
                            #'withproject' : argDict.get('withproject'),
		            'timeout'     : argDict.get('timeout'),
		            'station'     : argDict.get('station'),
                            'group'       : argDict.get('group'),
#                            'nodeName'    : argDict.get('node')
                            'nodeName'    : 'sam3.farm.particle.cz', # ugly hack!!!
			 }
      
      self._jobFileDsId   = argDict.get('jobFilesDatasetId')
      self._jobFileDsName = argDict.get('jobFilesDatasetName')
      self._mergeDsId     = argDict.get('mergeDatasetId')
      self._mergeDsName   = argDict.get('mergeDatasetName')
      self._mergeDim      = argDict.get('dim')
      self._mergeFileList = argDict.get('filename')
      self._eventsPerFile = int(argDict.get('eventsPerFile'))
      self._d0ReleaseVer  = argDict.get('appversion')
      self._jobNumber     = argDict.get('jobNumber')  
      self._incoming_fcp   = argDict.get('incoming-fcp')
      self._incoming_fcp_data = argDict.get('incoming-fcp-data')
      self._incoming_node_data = argDict.get('incoming-node-data')
      self._outgoing_fcp  = argDict.get('outgoing-fcp') 
      self._outgoing_path = argDict.get('outgoing-path')
      self._facilityname  = argDict.get('facilityname')
      
      if not (self._jobNumber == None):
          self._jobNumber = int(self._jobNumber)

      if ( self._incoming_node_data == None ):
         self._incoming_node_data = self._commonargs['nodeName']
   
      if ( self._incoming_fcp_data == None ):
         self._incoming_fcp_data = self._incoming_fcp

      # stage and setup job files
      if self._jobFileDsId or self._jobFileDsName :
         self.stageJobFiles()

      # stage files to be merged
      cwd = os.getcwd()
      inDir = os.path.join(cwd, "unmerged_files")
      fileList = self.stageFilesToBeMerged(inDir)

      if (fileList == None) or not len(fileList):
         raise Exception("No files to be merged in file list")
 
      # verify file transfers and create input list file
      unmergedFiles = "unmerged_files.txt"
      fileList = self.createInputFileList(inDir, fileList, unmergedFiles)
      
      # save file list as it'llbe bassed by refernce and
      # createOutputFileList makes it []
      # also list1 = list2 is by reference
      fileListSav = []
      for f in fileList:
         fileListSav.append(f)
      # end for

      print fileListSav
      # Create Output File List
      mergedFiles = "merged_files.txt"	
      outputDirPrefix = "%s/%s" % (cwd, "dest")
      outputDir = os.path.join(outputDirPrefix, "copyd0om")
      mergedFilesList = self.createOutputFileList(outputDir, \
                                                  fileList, mergedFiles)
      os.makedirs(outputDir)
       
      # Gather the list of files created as a part of the custom_out*.tar.gz
      shipOutput = """cd %s; ls -laR %s* > ./created_files.txt; tar cvfz $OUTPUT_FILE `find  . -name *log` ./created_files.txt `find . -name core*` `find . -name events.copy*` `find . -name macro*` `find . -name import_*py`""" % (cwd, cwd)

      #create the macro
      macroFile = self.createMacro(self._d0ReleaseVer,unmergedFiles,\
                         mergedFiles,outputDir, self._eventsPerFile,\
                         fileListSav, mergedFilesList)

      try:
         self.updateXMLDB("Launching d0runjob")
         self.d0runjob(macroFile)

         d0runjob_jobdir = "%s-1" % (os.environ['JIM_GLOBAL_JID'])
         d0runjob_exit_code_file = "%s/%s/exit_code" % (cwd, d0runjob_jobdir) 
         copyd0om_exit_code_file = "%s/%s/D0Copydoom-%s-%s/exit_code" % (
                                        cwd, \
                                        d0runjob_jobdir, \
                                        self._d0ReleaseVer, \
                                        d0runjob_jobdir)

         try:
             f1 = open(copyd0om_exit_code_file)
             copyd0om_exit_code = f1.read()[0]
             self.updateXMLDB("D0 Executable exited with code %s" % copyd0om_exit_code)
         except:
             self.updateXMLDB("D0 Executable exit code unavailable")
         
         try:
             f1 = open(d0runjob_exit_code_file)
             d0runjob_exit_code = f1.read()[0]
             self.updateXMLDB("d0runjob exited with code %s" % d0runjob_exit_code)
         except:
             self.updateXMLDB("d0runjob exit code unavailable")

         # VERIFY OUTPUT HERE
         self._print.warning("NOT YET VERIFYING COPYD0OM OUTPUT")
             
         outFiles = os.listdir(outputDir)
         self._print.info("Discovered following files in directory %s: %s" \
                          %(outputDir,str(outFiles)))
   
         if ( len(outFiles) > 1 ):
            raise Exception("D0 run job merged into more than one outputfile")

         if ( len(outFiles) == 0 ):
            raise Exception("D0 run job failed to merge input files")

         initialMD = self.computeParents(outFiles[0], fileList)
         self._print.info("Initial metadata generated")
         print "==================================="
         print initialMD
         print "==================================="
         
         self.generateMDandStore(outputDir, initialMD)
         print fileListSav
      except:
         self._print.info("Finding and tarring up the output: %s" % shipOutput)
         os.system(shipOutput)
         raise

      self._print.info("Finding and tarring up the output: %s" % shipOutput)
      os.system(shipOutput)
      os.system("******************************************")
      os.system("ls -latrR .")
      os.system("******************************************")
      self.cleanup(fileListSav)

   ## ==== end of init
   def getFiles(self,args):
      try:
         self.resetEnv("SAM_FCP_QUEUE_NAME",self._incoming_fcp_data)
 
         myArgs = args.copy()
         myArgs['nodeName'] = self._incoming_node_data

         dhglite = samProject.dhGetLite()
         return dhglite.getImplementation()(myArgs,[])
      except Exception,ex:
         self._print.error("Failed to fetch dataset")
	 raise ex
   ## ==== end of getFiles

   def resetEnv(self,envname,value):
      if ( value ):
         os.environ[envname] = value 
      else:
         if ( os.environ.has_key(envname) ):
             del os.environ[envname]

   def updateXMLDB(self,string):
      """Update the JIM XML DB"""
      self._print.info("updateXMLDB::Setting up the environment")
      try:
         gjid = os.environ['JIM_GLOBAL_JID']
         cjid = os.environ['JIM_CLUSTER_JID']
         ljid = os.environ['JIM_LOCAL_JID']
         url  = os.environ['XMLDB_SERVER_URL']
      except Exception, ex:
         self._print.warning("%s env variable missing" % ex)
         self._print.warning("Cannot update XML database")
         return

      self._print.info("updateXMLDB:: Making the upgrade")
      try:
         self._print.info("Updating XML database")
         from xmldb_client import idmgr
         jobCollection = "/db/jobs/%s" % (gjid)
         db = idmgr.idmgr(url, jobCollection)
	 updateStr = '<progress created=\"%s\" time_epoch=\"%s\">%s</progress>' %(db.getCurTime(), db.getCurEpochTime(), string)
         self._print.info("updateXMLDB:: %s" % updateStr)
         db.UpdateLocal("append", gjid, cjid, ljid, updateStr)
      except Exception, ex:
         self._print.warning(ex)
         self._print.warning("Could not Update XML Database... will continue")
         pass
      self._print.info("updateXMLDB:: Done with the upgrade")


   def untar(self,pattern):
      cmd = "tar xzf %s && rm -f %s" % (pattern,pattern)
      status,stdouterr = commands.getstatusoutput(cmd)
      if status != 0:
	 raise Exception("untar failed %s" % stdouterr)
   ## ==== end of untar


   def getD0runjobDir(self, path):
      import os
      list = os.listdir(path)
      pattern = "d0runjob"

      for i in list:
         if i.startswith(pattern):
            return i

      raise Exception("Unable to get the D0_RUNJOB_DIR")
   ## ==== end of getD0runjobDir

   def stageJobFiles(self):
      try :
         self._print.info("Staging job files")
	 update_str = "Staging Job Files"
	 self.updateXMLDB(update_str)
         cwd = os.getcwd()
         stream = Fetcher.FetchStream(\
                      cwd,\
                      Fetcher.getFileIdList(self._jobFileDsId, \
                                            self._jobFileDsName),\
                      self._commonargs)
         while 1:
            chunk = stream.fetch(5)
            if ( chunk == None ):
               break

         self.updateXMLDB("Finished staging job files")
         self._print.info("Unpacking d0 code")
	 update_str = "Unpacking d0 code"
     	 self.updateXMLDB(update_str)
         self.untar("d0_*.tar.gz")
         os.environ['SRT_DIST'] = "%s/mcc-dist" % cwd
         self._print.info("Unpacking d0runjob")
         # P.V. hack to use IPv6 patched version of d0runjob
         for fn in os.listdir('.'):
             if not os.path.isfile(fn): continue
             if not fn.startswith('d0runjob'): continue
             fn_ipv6 = "/home/samgrid/%s.ipv6" % fn
             if not os.path.isfile(fn_ipv6): continue
             self._print.info("Using IPv6 patched version of d0runjob (%s)" % fn_ipv6)
             import shutil
             shutil.copy(fn_ipv6, fn)
	 update_str = "Unpacking d0runjob"
	 self.updateXMLDB(update_str)
         self.untar("d0runjob*.tar.gz")
         os.environ['D0_RUNJOB_DIR'] = "%s/%s" % (cwd,self.getD0runjobDir(cwd))
      except Exception,ex :
         self._print.error("Failed to stage job files")
	 update_str = "Failed to stage job files"
         self.updateXMLDB(update_str)
       	 raise ex
   ## ==== end of setupJobFiles

   def getAllFileIdList(self):

      try :
         if ( self._mergeDsName ): 
            fileNameList = self.getFileListByName(self._mergeDsName)
         else: 
           if ( self._mergeDsId ):
              fileNameList = self.getFileListById(self._mergeDsId)
           else:
              raise Exception("Must give either merge ds name or snapshot id")

	 self._print.info("Mapping file names to file ids")
         facilityNames = []
         for fname in fileNameList:
            siteName = self.getFacilityNameOf(fname)
            if ( siteName == None ):
               siteName = "ZZZZZZZZ" 
            facilityNames.append( (siteName, fname) )

         facilityNames.sort()

         self._print.info("facilityNames:"+str(facilityNames))

         fileIdList = []

         if ( len(facilityNames) == 0 ):
            return fileIdList

         if self._jobNumber: 
            self._print.info("jobNumber is %s. Calculating events to merge" % self._jobNumber)
            endEvent   = self._jobNumber * self._eventsPerFile
            startEvent =  endEvent - self._eventsPerFile + 1
            self._print.info("Will merge events from %d to %d" % (startEvent,endEvent))

            fileIdNameList = []
            for facilityName,fname in facilityNames:
               fileIdNameList.append((self._samMDCache.get(fname)["fileId"],fname))

            results = findFileByEvents(self._print,fileIdNameList,self._jobNumber,self._eventsPerFile,0)

            if ( len(results) == 0 ):
               return fileIdList

            self._print.info("Sub selected list of files for event range "+str(results))

            siteName = self.getFacilityNameOf(results[0]['fileName'])
            for result in results:
               fname = result['fileName']
               if ( self.getFacilityNameOf(fname) == siteName ): 
                  #use the first facility in the sorted list
                  fileIdList.append(result['fileId']) 
         else:
            self._print.info("jobNumber is not available")
            siteName = facilityNames[0][0]
            for facilityName,fileName in facilityNames:
               if ( siteName == facilityName ):
                   #use the first facility in the sorted list
                   fileIdList.append(self._samMDCache.get(fileName)["fileId"])
             
         # end for
         self._print.info("There are %d files in the dataset" % len(fileIdList))          
         print fileIdList
	 return fileIdList
      except Exception,ex:
         self._print.warning("Failed to get list of ALL files in dataset")
         self._print.warning(str(ex))
	 return []
         	    

   def stageFilesToBeMerged(self,inDir):
      cwd = os.getcwd()
      self._print.info("Staging files to be merged")
      update_str = "Staging files to be merged"
      self.updateXMLDB(update_str)
      try :
         os.system("[ -d %s ] || mkdir -p %s" %(inDir,inDir))
         os.chdir(inDir)
      except Exception,ex :
         self._print.error("Could not create/change directory %s" % inDir)
	 raise ex

      if (self._mergeDsId == None) and (self._mergeDsName == None) :
         if not (self._mergeDim == None) :
	    self._mergeDsName = self.createDataset(self._mergeDim)
	 elif not (self._mergeFileList == None):
	    self._mergeDsName = self.createDataset("file_name %s" %(self._mergeFileList))
	 else :
	    raise Exception("No way to fetch files to be merged")
            update_str = "No way to fetch files to be merged"
            self.updateXMLDB(update_str)


      args = self._commonargs
      
      filesToRetrieve = self.getAllFileIdList()

      if ( len(filesToRetrieve) == 0 ):
         raise Exception("Could not determine file list from the event range to merge")

      args['fileList'] = filesToRetrieve


      fileNameList = []
      fileIdList   = []
      results      = []
      # stage files for merging
      try :
         fileIdList,results = self.getFiles(args)
         if not len(fileIdList):
	    raise Exception("dhGetLite returned blank file list")

         if not len(results):
            self._print.info("Mapping file ids to file names")

            for fid in fileIdList:
               fileName = self.mapFileIdToName(fid)
               fileNameList.append(fileName) 						       
         else :
	    # use results returned by dhgetlite 
	    for res in results :
	       if not(res['truncated']):
	          fileNameList.append(res['fileName'])
	       else :
	          # ignore truncated files to avoid a file having 2 merged tmbs as children
		  self._print.warning("Ignoring truncated file %s" % res['fileName'])
		  os.unlink(res['fileName'])

         self._print.info("There are %d files to be merged" % len(fileNameList))
         fileNameList.sort()
         os.chdir(cwd)
         return fileNameList
      except Exception,ex :
         print ex
         self._print.error("Failed to stage files to be merged") 	 
         update_str = "Failed to stage files to be merged"
         self.updateXMLDB(update_str)


	 raise ex
      
   ## ==== end of getFilesToBeMerged

   def createDataset(self,dim):
      self._print.write("Creating dataset for dim=%s" % dim)  
      user_name = ""
      if os.environ.has_key('SAM_USER_NAME'):
         user_name = os.environ['SAM_USER_NAME']
  
      os.environ['SAM_USER_NAME'] = "sam"
      try :
         uniq_def_name = "jim" + time.strftime("%d%m%Y%H%M%S", time.gmtime()) + \
                           str(random.randint(0,1027))
         sam.createDatasetDefinition( defName =  uniq_def_name, group = "dzero", dim=dim)

      except Exception,ex :
         self._print.error("Failed to create dataset")
	 raise ex

      if len(user_name): os.environ['SAM_USER_NAME'] = user_name
      print "done:", uniq_def_name
      return uniq_def_name
   ## ==== end of createDataset

   def getFileListByName(self,defname,furtherConstraint=""):
     return self.translateConstraints("__SET__ %s %s" % (defname,furtherConstraint))
   ## ==== end of getFileListByName

   def getFileListById(self,defid,furtherConstraint=""):
     return self.translateConstraints("DATASET_ID %s %s" % (defid,furtherConstraint))
   ## ==== end of getFileListByName

   def translateConstraints(self,dim):
      self._print.write("Translating constraints: %s... " % dim)
      try :
         res =  sam.translateConstraints(dim=dim)
      except Exception,ex :
         self._print.error("Failed to translate dimension %s" % dim)
         raise ex

      print "done"
      return map(lambda s: s["fileName"] , res )

   ## ==== end of translateConstraints

   def getFileListBySnapId(self,snapid):
      fileList = []
      fileDataList = sam.translateConstraints(dim=snapid)
      for file in fileDataList:
         fileList.append(file["fileName"])
       # end for

      return fileList
   ## ==== end of getFileListBySnapId

   def mapFileIdToName(self,fileId):
      try :
         return sam.getMetadata(fileId=fileId)["fileName"]
      except Exception,ex:
         self._print.error("Could not get file name for fileid %d" % fileId)
	 raise ex
   ## ==== end of mapFileIdToName


   # This function will take as input either a list of files,dataset name
   # or a dim query. It will get all the files for the input from SAM
   # and returns the list of files staged
   def compareLists(self,l1, l2):
      """Returns: triple containing (intersect(l1, l2), l1-l2, l2-l1)"""
      l1.sort()
      l2.sort()
      intersect=[]
      diff1=[]
      diff2=[]
      i1=0
      i2=0
      while i1 < len(l1) and i2 < len(l2):
         if l1[i1]==l2[i2]:
            intersect.append(l1[i1])
            i1 = i1 + 1
            i2 = i2 + 1
         elif l1[i1] < l2[i2]:
            diff1.append(l1[i1])
            i1 = i1 + 1
         else:
            diff2.append(l2[i2])
            i2 = i2 + 1
         pass
      # end while

      if i1 <= len(l1):
         diff1 = diff1 + l1[i1:]
      elif i2 <= len(l2):
         diff2 = diff2 + l2[i2:]
      else:
         assert(i1 == i2)

      return intersect, diff1, diff2
   # ==== end of compareLists

   # This function creates a text file containig all the input files
   # to be fed to d0runjob
   def createInputFileList(self,inDir, filelist, filename):
      if len(filename) == 0:
         raise Exception("Invalid filename specified for input file")

      actualFiles = os.listdir(inDir)
      actualFiles.sort()
      if not len(actualFiles) :
         raise Exception("No files under %s" % inDir)
   
      if actualFiles != filelist:
         self._print.warning("fetched files do not match expectations!")
         finalList, diff1, diff2 = self.compareLists(filelist, actualFiles)
         for f in diff1:
            self._print.error("file %s could not be delivered!" % f)
         for f in diff2:
            self._print.error("file %s did NOT come from SAM!" % f)
      else:
         finalList = filelist

      try :
         file = open(filename, 'w')
         for f in finalList :
            file.write("%s/%s\n" %(inDir, f) )
         file.close()
      except Exception,ex:
         self._print.error("Failed to write InputFileList")    
         raise ex
	 
      return finalList
   # ==== end of createInputFileList


   # This function creates a text file that contains the names
   # of the output files to be produced by d0runjob
   def createOutputFileList(self,outputDir, filelist, filename) :
      if len(filename) == 0:
         raise Exception("Invalid filename specified for output file")

      cleanupCmd = "rm -rf %s" % outputDir
      self._print.info("Executing %s" % cleanupCmd)
      os.system(cleanupCmd)
   
      totalEvents = 0
      currentWorkingDir = os.getcwd()
      self._print.info("Collecting total number of events in the dataset")

      # get the total number of events in the dataset
      totalEvents = 0
      for f in filelist:
         self._print.info("Getting metadata of file %s" %f)
         numEvents = self.getNumEvents(f)
         self._print.info("num events = %d" % numEvents)
         totalEvents = totalEvents + numEvents
      # end for

      if totalEvents == 0:
         raise Exception("There are no event in the dataset")

      self._print.info("Total number of events %d put in 1 file" % \
                                totalEvents )


      template = string.split(filelist[0],"_")
      template = template[:len(template)-1]
      template = string.join(template,'_')

      reObj = re.compile("NumEv-[0-9]*")

      eventsLeft = totalEvents;

      outFileList = []
      outFileList1 = []
      template = reObj.subn("NumEv-%d" % eventsLeft, template)[0]     	  
         
      self._print.info("File name template: %s" % template)
	 
      fname = "%s-%d%d-JIM_MERGE_%d" %(template, int(time.time()), os.getpid(),0)
      self._print.info("Appending filename %s" % fname)
      outFileList.append("%s\n" %(fname))      
      outFileList1.append("%s" %(fname))      

      # end while
      try :   
	 f = open(filename, "w")
         for i in outFileList: f.writelines("%s/%s" %(outputDir,i))
         f.close()
      except Exception,ex :
         self._print.error("Failed to create output file list")
	 raise ex
	 
      self._print.info("Generated file list: %s" % str(outFileList) )
      sys.stdout.flush()
      return outFileList1
   ## ==== end of createOutputFileList


   # This fucntion creates a macro for running a merging job
   def createMacro(self, d0release, unmergedFiles, mergedFiles, destDir, maxEventsPerFile, unmergedFilesList, mergedFilesList):

      working_dir = os.getcwd()
      srt_dist = os.environ['SRT_DIST']
      srt_dist_dir = srt_dist + "/releases/" + d0release

      for i in range(0, len(unmergedFilesList)):
          try:
              print "-> FINDING: %s in %s " % (unmergedFilesList[i],working_dir)
              os.system("find . -name %s" % (unmergedFilesList[i]))
          except:
              print "======= ERROR FINDING %s" % (unmergedFilesList[i])
              pass
          unmergedFilesList[i] = "%s/unmerged_files/%s" % (working_dir,\
                                                           unmergedFilesList[i])
      unmergedFiles_list = str(unmergedFilesList)
      for i in range(0, len(mergedFilesList)):
          mergedFilesList[i] = os.path.join(destDir,mergedFilesList[i])
      mergedFiles_list = str(mergedFilesList)
      gjid = os.environ['JIM_GLOBAL_JID']

      print "-------------------> Additional Info"
      print "%s %s %s %s %s %s" % (working_dir, srt_dist, srt_dist_dir,\
                                   unmergedFiles_list, mergedFiles_list, gjid)



      macro="""
import os

locald0 = os.path.expandvars("%s")

from d0rjcore.d0userinterface import createUserInterface
d0rj = createUserInterface(
    JobName = "%s",
    JobCacheArea = "%s",
    OptionalMonitors = ['jim'],
)

d0copydoom = d0rj.attachExecutable("D0Copydoom")
d0copydoom["InputFiles"] = %s
d0copydoom["OutputFiles"] = %s
d0copydoom["ProjectVersion"] = "%s"
d0copydoom["DistroDirectory"] = "%s"

# Register general job manager with BashJobOperator in order to produce
# the required job and task scipts
bashjob = d0rj.attachConfigurator("BashJobOperator")
bashjob.registerManager(d0rj.getManager())

prov = d0rj.attachConfigurator("ProvenanceProcessor")
prov.registerConfigurator(d0copydoom)

run = d0rj.attachConfigurator("RunInteractive")
run.registerManager(d0rj.getManager())

for i in range(0, 1):
  d0rj.run()

""" %(srt_dist_dir, gjid, working_dir, unmergedFiles_list, mergedFiles_list, d0release, srt_dist)

      macroFile ="merge.macro"
      try :
         f = open(macroFile, 'w')
         f.write(macro)
         f.close()
      except Exception, ex :
         self._print.error("Failed to write macro file")
         raise ex

      self._print.info("Macro dumped in file %s" % macroFile)
      update_str = "Generated Macro"
      print "------------------------------------"
      print "         %s            " % update_str
      print "------------------------------------"
      print "%s" % macro
      print "------------------------------------"

      self.updateXMLDB(update_str)
      return macroFile

   ## ==== end of createMacro      

   # call d0runjob
   def d0runjob(self,macroFile):
      # Setup the environment for d0runjob
      os.environ['PYTHONPATH'] = os.environ['PYTHONPATH'] + ":" + os.environ['D0_RUNJOB_DIR'] + "/Python"
      os.environ['SHAHKAR_FACTORY_EXT'] = os.environ['D0_RUNJOB_DIR'] + "/xml/d0factory.xml"

      try :
         if not os.environ['D0_RUNJOB_DIR']:
            raise Exception("Need D0_RUNJOB_DIR env variable set or specify jobfiles dataset")

         self._print.info("Invoking d0runjob to merge files")
         cmd = "python %s" %(macroFile)
         sys.stdout.flush()
         print "==============================================\n"
         print os.environ
         print "==============================================\n"
         print "Executing %s" % (cmd)
         status = os.system(cmd)
         print "Exit code = %d" % status
         print "\n=============================================="

         if status != 0:
            raise Exception("d0runjob returned an invalid status code")
      
      except Exception,ex:
         self._print.error("d0runjob failed !")
	 raise ex	    
   ## ==== end of d0runjob

   def getNumEvents(self,infile):
      return self._samMDCache.get(infile)["eventCount"]
   ## ==== end of getNumEvents

   def computeParentsOneFile(self,infiles ):
      #The output file's last event number
      result = {}
      result['parents'] = []
      result['firstEvent'] = 1
      numEvents = 0
      while 1:
         if len(infiles) == 0:
            result['lastEvent'] = numEvents
            break
         else :               
            infile = infiles.pop(0)
            # For MC parents of merged thumbnail should be d0sim files and not
            # not unmerged thumbnails. This is the request from Joel
            result['parents'] = result['parents'] + self.getParents(infile)
            numEvents += self.getNumEvents(infile)
       
      # end while        
      return  result 
   # ==== end of computeParentsOneFile

   def computeParents(self, outFile, inFiles):

      result = {}
      if not len(inFiles):
            raise Exception ("Too few in files")

      oneRes = \
                self.computeParentsOneFile\
                ( inFiles )
      result[outFile] = oneRes
      # end while
      return result
   # ==== end of computeParents


   def getParents(self,fileName):
      return map( lambda x: x.name , self._samMDCache.get(fileName).get("parents") )
   ## ==== end of getParents

   def makeSamDbTime(self,secs):
      return time.strftime("%m/%d/%Y %H:%M:%S", time.gmtime(secs))
   ## ==== end of makeSamDbTime

   def makeDict(self,samKeys, replacementMap):
      """Converts SAM keywords into a dick replacing some values"""
      res = {}
      for k in samKeys:
         if not res.has_key(k.category): res[k.category] = {}
         self._print.info('HOCKEY %s' % k.keyName)
         if replacementMap.has_key(k.keyName):
            value = replacementMap[k.keyName]
         else:
            value = k.keyVal.v
         res[k.category][k.keyName] = value
       # end for
      return res
   ## ==== end of makeDict

   def getParentsOfParents(self,parents):
      result = []
      for p in parents:
         for gp in self.getParents(p):
            if gp not in result: result.append(gp.name)
      return result
   ## ==== end of getParentsOfParents


   def generateMD(self,fileName, outDir, basicMD, samMD):

      self._print.info("SAM Template MD :"+str(samMD))

      fileStruct = os.stat(os.path.join(outDir, fileName))
      fileSize   = fileStruct[6]/1024
      fileCreateTime = fileStruct[9]
      fileModifiedTime = fileStruct[8]
			 
      header = """from SamFile.SamDataFile import ImportedSimulatedFile
from SamStruct.SamSize import SamSize
from SamStruct.CRC import CRC
from SamStruct.NameOrIdList import NameOrIdList
from SamUtility.CaseInsensitiveDictionary import CaseInsensitiveDictionary
from SamStruct.SamTime import SamTime
from SamStruct.ApplicationFamily import ApplicationFamily
from SamStruct.Params import Params
from SamStruct.RunDescriptorList import RunDescriptorList
from SamStruct.RunDescriptor import RunDescriptor
from SamStruct.ParamValue import ParamValue
from SamStruct.DataType import DataType

"""
      newMDStr= header + "newMD = "+str(samMD) 
     
      exec newMDStr

      from SamStruct.NameOrIdList import NameOrIdList
      from SamStruct.SamSize import SamSize 
      from SamStruct.SamTime import SamTime

      newMD["fileName"] = fileName
      newMD["parents"] = NameOrIdList(basicMD["parents"])
      newMD["firstEvent"] = basicMD["firstEvent"]
      newMD["lastEvent"] = basicMD["lastEvent"]
      newMD["eventCount"] = basicMD["lastEvent"] - basicMD["firstEvent"] + 1
      #newMD["eventCount"] = basicMD["totalevents"]
      newMD["fileSize"] = SamSize(str(fileSize)+"K")
      newMD["startTime"] = SamTime(fileCreateTime)
      newMD["endTime"] = SamTime(fileModifiedTime)
      newMD["dataTier"] = "thumbnail"
      newMD["params"]["global"]["facilityname"]=self.getFacilityName()

      if (os.environ.has_key('JIM_GLOBAL_JID')):
          newMD["params"]["global"]["jimglobaljobid"]=os.environ['JIM_GLOBAL_JID']
          newMD["params"]["global"]["jimclusterjobid"]=os.environ['JIM_CLUSTER_JID']
          newMD["params"]["global"]["jimlocaljobid"]=os.environ['JIM_LOCAL_JID']
          newMD["params"]["global"]["producedbyname"]="SAM-Grid"

      if (newMD.get('params')):
          if (newMD.get('params').has_key('global')):
              if (newMD.get('params').get('global').has_key('runjobversion')):
                  del(newMD.get('params').get('global')['runjobversion'])

      newMD["crc"] = sam.calculateFileCrc(path=os.path.join(outDir, fileName))
      self._print.info("SAM Generated MD :"+str(newMD))

      mdFileName = "%s/import_%s.py" % (outDir, fileName)
      try :   
	 mdFile = open(mdFileName, 'w')
         mdFile.write(header + "t=" + str(newMD))
         mdFile.close()
      except Exception, ex :
         self._print.error("Failed to write metadata file")
	 raise ex
	 
      return mdFileName
   ## ==== end of generateMD


   def generateMDandStoreOne(self,fileName, outDir, basicMD, samMD):
      mdFileName = self.generateMD(fileName, outDir, basicMD, samMD)
      # Declare the metadata and then do a sam store
      self._print.info("Trying to store the merged file with metadata in %s " % mdFileName)
      sys.stdout.flush()
      

      try:
          self.resetEnv("SAM_FCP_QUEUE_NAME",self._outgoing_fcp)
          filepath = os.path.join(outDir,fileName)
          from Sam import sam

          if ( self._outgoing_path ):
              transferRequestId = sam.storeFile(sourceFile=filepath, \
                                                waitForCompletion=1,\
                                                temporaryTransferLocation=self._outgoing_path,\
                                                retryMaxCount=3,\
                                                descriptionFile=mdFileName)
          else:
              transferRequestId = sam.storeFile(sourceFile=filepath, \
                                                waitForCompletion=1,\
                                                retryMaxCount=3,\
                                                descriptionFile=mdFileName)
          self.updateDB("stored",fileName)
      except Exception,ex:
          traceback.print_exc(file=sys.stderr)
          self._print.error("Failed to store file with metadata in %s" % mdFileName)
          raise "Failed to store file with metadata in %s" % mdFileName
   ## ==== end of generateMDandStoreOne

   def updateDB(self,state, filename):
      """Update the JIM XML DB"""
      try:
         gjid = os.environ['JIM_GLOBAL_JID']
         cjid = os.environ['JIM_CLUSTER_JID']
         ljid = os.environ['JIM_LOCAL_JID']
         url  = os.environ['XMLDB_SERVER_URL']
      except Exception, ex:
         self._print.warning("%s env variable missing" % ex) 
         self._print.warning("Cannot update XML database")
         return
 
      try:
         self._print.info("Updating XML database")
         from xmldb_client import idmgr
         jobCollection = "/db/jobs/%s" % (gjid)
         db = idmgr.idmgr(url, jobCollection)
         updateStr = '<output_file name=\"%s\" time=\"%s\" time_epoch=\"%s\" state=\"%s\"/>' % \
                       (filename, db.getCurTime(), db.getCurEpochTime(), state)
         db.UpdateLocal("append", gjid, cjid, ljid, updateStr)
      except Exception, ex:
         self._print.warning(ex) 
         self._print.warning("Could not Update XML Database... will continue")
         pass
   ## ==== end of updateDB

   
   def generateMDandStore(self,outDir, initialMD):
      """Generate SAM MD and store outFiles to SAM, using initial MD.
         As a template, use MD for one of the parents."""
      outFiles = initialMD.keys()
      #anUnmergedTMB = initialMD[outFiles[0]]['parents'][0]
      sim = initialMD[outFiles[0]]['parents'][0]
      anUnmergedTMB = sam.getFileDescendants(fileName=sim)[0].getFileName()
      templateMD = self._samMDCache.get(anUnmergedTMB)
      for f in outFiles:
         self.generateMDandStoreOne(f, outDir, initialMD[f], templateMD)
      # end for
   ## ==== end of generateMDandStore


   def cleanup(self,in_tmbList):
      # Get only the filenames. Chop off the full path 
      tmbList = []
      for i in range(0, len(in_tmbList)):
          tmbList.append((in_tmbList[i].split('/')[-1]))

      self._print.info("Cleaning up source files from SAM")
      print tmbList  
      if not len(tmbList):
         self._print.warning("No thumbnails to clean")
	 return
      # end if
      
      # tmbs that have been merged
      tmbs2remove = self.getTmbsToRemove(tmbList)
      if not len(tmbs2remove):
         self._print.warning("There are no thumbnails to remove")
         return
         
      self._print.info("Removing files %s" % str(tmbs2remove))
      for tmb in tmbs2remove:
         self.removeTmb(tmb)
      # end for 	 
   ## ==== end of cleanup
   
   def removeTmb(self,tmb):
      try :
         locs = sam.locate(args=[tmb])
	 ("Removing locations for file %s" % tmb)
         for loc in locs:
            self._print.info("Removing location: %s" % loc)
            sam.eraseFileLocation(fileName=tmb, replicaLocation=loc.fullPath )
	 # end for 					
      except Exception,ex:
         self._print.warning(ex)
	 self._print.warning("Error while removing locations for file %s" % tmb)
	 pass
   ## ==== end of removeTmb   

   def getTmbsToRemove(self,tmbList):
      result = []
      for tmb in tmbList:
         if self.hasBeenMerged(tmb) and  not self.isMerged(tmb):
	    result.append(tmb)
      # end for	    
      return result
   ## ==== end of getTmbsToRemove
      
   def isMerged(self,tmb):
      # false:0 true:1
      parents = self.getParents(tmb)
      if len(parents) > 1:
         self._print.warning("File %s IS merged" % (tmb))
	 return 1
      else:
         return 0
   ## ==== end of isMerged
        
   def hasBeenMerged(self,tmb):
      # false:0 true:1 
      tmbMd = self._samMDCache.get(tmb)
      # for each parent look for a child != this file
      for parent in tmbMd.get("parents"):
         children = sam.getFileDescendants(fileName=parent)
         self._print.info("Children of file %s : %s" % (parent,str(children)) )
	 for c in children :
            child = c.getFileName()
	    if not (child == tmb) and self.isThumbnail(child):
	       # this file has a sibling tmb
	       self._print.info("File %s has a sibling %s that IS a tmb" % (tmb,child))
	       self._print.info("File %s HAS BEEN merged" % tmb)
	       return 1
	 # end for      
      # end for 
      self._print.warning("File %s has NOT been merged" % tmb) 
      return 0 	       
      # end for
   ## ==== end of hasBeenMerged


   def getChildren(self,file):
       raise "Not implemented"

   ## ==== end of getChildren
   
   # check to see if file is a tmb and has a valid location in SAM
   def isThumbnail(self,file):
      tmbMd = self._samMDCache.get(file)
      if tmbMd["dataTier"] == "thumbnail":
         locs = sam.locate(args=[file])
         if len(locs):
	    return 1
	 else:
            return 0	       
      else :
         return 0  	 
   ## ==== end of isThumbnail

   ## === retrieve edg site name if available
     
   def getFacilityName(self):
       return self._facilityname

   def getFacilityNameOf(self,fileName):
       for ( keyName,keyVal ) in self._samMDCache.get(fileName)["params"]["global"].items():
          if ( keyName == "facilityname" ):
             return keyVal
       return None

   ## ==== end of getFacilityName

def findFileByEvents(log,fileIdNameList,jobNumber, numEvents, numEventsToSkip):
               eventsToSkip = (jobNumber - 1) * numEvents + numEventsToSkip

               jobEventRange_Start = eventsToSkip + 1
               jobEventRange_Stop  = eventsToSkip + numEvents

               log.info("JOB EVENT RANGE: %d to %d" %\
                             (jobEventRange_Start,jobEventRange_Stop))

               eventsSeenInDataset = 0
               results = []

               fileNumber = 1
               for f,fileName in fileIdNameList:
                   fileEvents = _samMDCache.get(fileName)["eventCount"]
                   fileEventRangeInDataset_Start = eventsSeenInDataset + 1
                   fileEventRangeInDataset_Stop  = eventsSeenInDataset + fileEvents

                   if jobEventRange_Start <= fileEventRangeInDataset_Stop\
                      and jobEventRange_Start >= fileEventRangeInDataset_Start:

                       results.append({
                                           'fileName'             : fileName,
                                           'fileEvents'           : fileEvents,
                                           'fileId'               : f,
                                           'fileNumber'           : fileNumber,
                                           'truncated'            : 0
                                        })
                   else:

                      if jobEventRange_Stop >= fileEventRangeInDataset_Stop\
                         and jobEventRange_Start <= fileEventRangeInDataset_Start:

                         results.append({
                                           'fileName'             : fileName,
                                           'fileEvents'           : fileEvents,
                                           'fileId'               : f,
                                           'fileNumber'           : fileNumber,
                                           'truncated'            : 0
                                        })
                      else:
                        if fileEventRangeInDataset_Start > jobEventRange_Stop:
                           break

                   # end if

                   eventsSeenInDataset = eventsSeenInDataset + fileEvents
                   fileNumber = fileNumber + 1
               # end for

               if not len(results):
                  # the range does not even start in the dataset
                  raise Exception("Job event range lies completely outside of this dataset")
               else :
                  return results
           
    
## ============== end of class MergeImplementation ============== ##
class MyTestLog:

  def info(self,msg):
     print msg
  def warning(self,msg):
     print msg

def testRanges():

  log = MyTestLog()

  from Sam import sam
  result = sam.translateConstraints(dim="__SET__ jim_merge_dataset_07052007120959620")

  fileIdNames = map( lambda x: (x['fileId'],x['fileName']), result)
  fileIdNames.sort()
  
  totEvs = 0
  assertMap = {}

  for i in range(1,46):
     result = findFileByEvents(log, fileIdNames ,i , 2000, 0)

     print "Current",totEvs

     for r in result:
        totEvs += r['fileEvents']
        if assertMap.has_key(r['fileName']):
           raise Exception("Attempt of duplicate merging")
        assertMap[r['fileName']] = 1
     print "Merging",totEvs


if __name__ == '__main__':
  _samMDCache = SamMDCache()

  print testRanges()

  fileName="tmb_p14.05.01_NumEv-200_gam+jets-incl_test_mcp14_SamgridFarm_12446-111463239016256-JIM_MERGE_0"
  fileList=["tmb_p14.05.01_NumEv-50_gam+jets-incl_test_mcp14_SamgridFarm_12446_8964345485363184804225134853",
            "tmb_p14.05.01_NumEv-50_gam+jets-incl_test_mcp14_SamgridFarm_12446_896434547281466204225134455",
            "tmb_p14.05.01_NumEv-50_gam+jets-incl_test_mcp14_SamgridFarm_12446_8964345472813156504225125951",
            "tmb_p14.05.01_NumEv-50_gam+jets-incl_test_mcp14_SamgridFarm_12446_8964345485363031904225134006",
           ]

  print getFacilityNameOf("tmb_p14.05.01_NumEv-50_gam+jets-incl_test_mcp14_SamgridFarm_12446_896434547281466204225134455")
  outDir="/home/abaranov/tmp/tmp/out"
  outputDir="/home/abaranov/tmp/tmp"
  eventsPerFile=200
  outputFiles = os.listdir(outDir)

  initialMD = computeParents(outputFiles, eventsPerFile, fileList)
  print "initialMD"
  print initialMD

  outFiles = initialMD.keys()
  anUnmergedTMB = initialMD[outFiles[0]]['parents'][0]
  print "anUnmergedTMB"
  print anUnmergedTMB
  
  templateMD = _samMDCache.get(anUnmergedTMB)
  
  mdFileName = generateMD(fileName, outputDir, initialMD[outFiles[0]], templateMD)
  print "mdFileName"
  print mdFileName
