#!/bin/sh

# UPS setups.
. ${SETUPS_DIR}/setups.sh

# Constants.
TRUE_=1
FALSE_=0

# Other variables.
otherProduct=sam_config
thisProduct=${UPS_PROD_NAME}
confDir=${UPS_THIS_DB}/${UPS_PROD_NAME}/${UPS_PROD_VERSION}
prodTemplateTableFile=${UPS_PROD_DIR}/conf/${UPS_PROD_NAME}_template.table
insertQualString="<INSERT_QUALIFIER>"

# Action and other ups parameters for other product.
upsAction=$1
shift 1
upsParameters="$@"


if [ "$upsAction" = "" ]; then
  echo "Usage: $0 <ups action>"
  exit 1
fi

# Given qualifier, return table file.
getTableFile() {
  qualifier=$1
  tableFile=${UPS_PROD_NAME}_${qualifier}.table
  echo ${tableFile}
}

# List all ups qualifiers.
upsListQualifiers() {
  product=$1
  ups list -acK qualifiers ${product} | sed 's/\"//g'
}

# Declare product qith new qualifier.
upsDeclare() {
  qualifier=$1
  tableFile=`getTableFile ${qualifier}`
  cmd="ups declare -r ${UPS_PROD_DIR} -M ${confDir} -m ${tableFile} \
    ${UPS_PROD_NAME} ${UPS_PROD_VERSION} -q ${qualifier} -0 -c"
  echo "Executing: $cmd"
  eval $cmd
}

# Create table file for a given qualifier.
createUpsTable() {
  qualifier=$1
  tableFile=`getTableFile ${qualifier}`
  fullTableFile=${confDir}/${tableFile}
  rm -f ${tableFile}
  sedCmd="s/${insertQualString}/${qualifier}/g"
  cmd="cat ${prodTemplateTableFile} | sed '$sedCmd' > ${fullTableFile}"
  echo "Executing: $cmd"
  eval $cmd
}

# Undeclare product with given qualifier.
upsUndeclare() {
  qualifier=$1
  cmd="ups undeclare ${UPS_PROD_NAME} ${UPS_PROD_VERSION} -q ${qualifier} -0"
  echo "Executing: $cmd"
  eval $cmd
}

# Remove table file for a given qualifier.
removeUpsTable() {
  qualifier=$1
  tableFile=`getTableFile ${qualifier}`
  fullTableFile=${confDir}/${tableFile}
  cmd="rm -f ${fullTableFile}"
  echo "Executing: $cmd"
  eval $cmd
}

# Compare two lists, return the items in list1 that are NOT in list2.
listDiff() {
  list1=$1
  list2=$2
  result=""
  for i in ${list1}; do
    inList2=${FALSE_}
    for j in ${list2}; do
      if [ "${i}" = "${j}" ]; then
        inList2=${TRUE_}
        break
      fi
    done
    if [ ${inList2} = ${FALSE_} ]; then
      result="${result} ${i}"
    fi
  done
  echo ${result}
}

# List all current product qualifiers.
#oldQualList=`upsListQualifiers ${thisProduct}`

# Configure other product.
ups ${upsAction} ${otherProduct} ${upsParameters}

# List all current product qualifiers.
#newQualList=`upsListQualifiers ${otherProduct}`

## Undeclare qualifiers that do not exist any more.
#undeclareOldQualList=`listDiff "$oldQualList" "$newQualList"`
#for q in ${undeclareOldQualList}; do
#  echo "Undeclaring ${UPS_PROD_NAME} with qualifier ${q}..."
#  upsUndeclare $q
#  removeUpsTable $q
#done
#
#declareNewQualList=`listDiff "$newQualList" "$oldQualList"`
#for q in ${declareNewQualList}; do
#  echo "Declaring ${UPS_PROD_NAME} with qualifier ${q}..."
#  createUpsTable $q
#  upsDeclare $q
#done

