#!/usr/bin/env sampy

import SAM
from SamFile.SamDataFile import SamDataFile, SamTime, SamSize

# build a list of sample files
ExampleMetadataList = []

# non-physics file: log files, etc., do not contain physics information
sampleNonPhysicsGenericFile = SamDataFile({
    SAM.attrFileName           : 'sample_nonphysics_generic_file',
    SAM.attrFileType           : SAM.DataFileType_NonPhysicsGeneric,
    SAM.attrFileFormat         : SAM.DataFileFormat_TARGZ,
    SAM.attrFileSize           : SamSize('41.05MB'),
    })
ExampleMetadataList.append(sampleNonPhysicsGenericFile)

# physics generic file: must contain a dataTier, but all other
#  physics-related information is optional.
samplePhysicsGenericFile = SamDataFile({
    SAM.attrFileName           : 'sample_physics_generic_file',
    SAM.attrFileType           : SAM.DataFileType_PhysicsGeneric,
    SAM.attrFileFormat         : SAM.DataFileFormat_Unknown,
    SAM.attrFileSize           : SamSize('564.79MB'),
    SAM.attrCrcValue           : '1136831670L',
    SAM.attrCrcType            : SAM.CRC_Adler32Type,
    SAM.attrDataTier           : 'unofficial_reco',
    })
ExampleMetadataList.append(samplePhysicsGenericFile)

# imported collider file: "real" (as opposed to "simulated")
# detector event data that has been brought into SAM
# without a SAM project (e.g., from the online system).
sampleImportedDetectorFile = SamDataFile({ 
    SAM.attrFileName           : 'sample_raw_data_file.raw',
    SAM.attrFileType           : SAM.DataFileType_ImportedDetector,
    SAM.attrFileFormat         : SAM.DataFileFormat_DSPACK,
    SAM.attrFileSize           : SamSize('129.08MB'),
    SAM.attrCrcValue           : '2081168408L',
    SAM.attrCrcType            : SAM.CRC_Adler32Type,
    SAM.attrDataTier           : 'raw',
    SAM.attrFirstEvent         : 18250272,
    SAM.attrLastEvent          : 18696852,
    SAM.attrEventCount         : 489,
    SAM.attrStartTime          : SamTime('31-Jan-2004'),
    SAM.attrEndTime            : SamTime('01-Feb-2004'),
    SAM.attrAppFamily          : 'datalogger',
    SAM.attrAppName            : 'datalogger',
    SAM.attrAppVersion         : '0.0',
    SAM.attrPhysicalDatastream : 'zero_bias',
    SAM.attrFilePartition      : 6,
    SAM.attrRunNumber          : 188900,
    SAM.attrRunType            : 'physics data taking',
    SAM.attrWorkGroup          : 'online daq',
    })
ExampleMetadataList.append(sampleImportedDetectorFile)

# derived collider file: "real" (as opposed to "simulated")
#  event data produced by running a SAM project over collider data.
sampleDerivedDetectorFile = SamDataFile({
    SAM.attrFileName           : 'sample_derived_detector_file',
    SAM.attrFileType           : SAM.DataFileType_DerivedDetector,
    SAM.attrFileFormat         : SAM.DataFileFormat_DSPACK,
    SAM.attrFileSize           : SamSize('336.06MB'),
    SAM.attrCrcValue           : '743552779L',
    SAM.attrCrcType            : SAM.CRC_Adler32Type,
    SAM.attrDataTier           : 'reconstructed',
    SAM.attrProcessId          : 3028031,
    SAM.attrFirstEvent         : 36556768,
    SAM.attrLastEvent          : 36614389,
    SAM.attrEventCount         : 1974,
    SAM.attrParents            : ['all_0000188743_044.raw'],
    SAM.attrWorkGroup          : 'dzero',
    SAM.attrRunNumber          : 14380,
    SAM.attrRunType            : 'physics data taking',
    SAM.attrPhysicalDatastream : 'zero_bias',
    })
ExampleMetadataList.append(sampleDerivedDetectorFile)

# simulated import file: simulated (monte carlo) data,
#  brought into SAM without a SAM project 
sampleImportedSimulatedFile = SamDataFile ({
    SAM.attrFileName           : 'sample_imported_simulated_file',
    SAM.attrFileType           : SAM.DataFileType_ImportedSimulated,
    SAM.attrFileFormat         : SAM.DataFileFormat_DSPACK,
    SAM.attrFileSize           : SamSize('5.79MB'),
    SAM.attrEventCount         : 250,
    SAM.attrDataTier           : 'generated',
    SAM.attrFirstEvent         : 1,
    SAM.attrLastEvent          : 250,
    SAM.attrAppFamily          : 'generator',
    SAM.attrAppName            : 'pythia',
    SAM.attrAppVersion         : 'p14.03.02',
    SAM.attrPhysicalDatastream : 'notstreamed',
    SAM.attrRunType            : 'monte carlo',
    SAM.attrRunNumber          : 369426,
    SAM.attrWorkGroup          : 'dzero',
    SAM.attrParams : {
      'global' : {
         'facilityname' : 'IN2P3',
         'groupname' : 'test',
         'originname' : 'IN2P3',
         'phase' : 'mcp14',
         'producedbyname' : 'nobody',
         'producedforname' : 'sonne',
         'requestid' : '11613',
         'workrequestid' : '04032065943' },
      'generated' : {
         'alpgenlinktoinput' : 'on',
         'appfamily' : 'generator',
         'appname' : 'pythia',
         'appversion' : 'p14.03.02',
         'cardfiledir' : 'bid',
         'cardfileversion' : 'v00-04-57',
         'chtechnipimass' : '85.0',
         'chtechnirhomass' : '175.0',
         'collisionenergy' : '1960.0',
         'd0messmaxevents' : '0',
         'd0release' : 'p14.03.02',
         'datatier' : 'generated',
         'decay' : 'ccbar',
         'etagt' : '-4.2',
         'etalt' : '4.2',
         'filesize' : '5929',
         'finishdate' : '02/01/2004 07:02:15',
         'firstevent' : '1',
         'frameworkrcpname' : 'runMCpythia.rcp',
         'frameworkrcppackage' : 'd0_mcpp_gen',
         'higgsmass' : '120.0',
         'hppmass' : '200.0',
         'inputfilesperfile' : '1',
         'kinmassgt' : '2.0',
         'kinmasslt' : '-1.0',
         'lastevent' : '250',
         'leptoquarkmass' : '250.0',
         'multiplepartoninteractions' : 'on',
         'numrecords' : '250',
         'onetoponlytop' : '0',
         'onetopprocess' : '13',
         'paramcategory' : 'Generated',
         'pdflibfunc' : 'CTEQ4L',
         'processid' : '12473',
         'production' : 'z',
         'ptgt' : '5.0',
         'ptlt' : '99999.0',
         'ranseed1' : '1069512701',
         'ranseed2' : '531447805',
         'startdate' : '02/01/2004 06:59:59',
         'synchronise' : 'off',
         'techniomega0mass' : '175.0',
         'technipi0mass' : '85.0',
         'technipi0primemass' : '85.0',
         'technirho0mass' : '175.0',
         'topmass' : '174.3',
         'totalevents' : '250',
         'usealpgen' : 'off',
         'usecomphep' : 'off',
         'useevtgen' : 'off',
         'useonetop' : 'off',
         'usepmcs' : 'off',
         'useqq' : 'off',
         'wprimemass' : '750.0',
         'zprimemass' : '700.0',
         },
      }
    })
ExampleMetadataList.append(sampleImportedSimulatedFile)

# simulated derived file: simulated (monte carlo) event data,
#  produced by running a SAM project over monte carlo data.
sampleDerivedSimulatedFile = SamDataFile({
    SAM.attrFileName            : 'sample_derived_simulated_file',
    SAM.attrFileType            : SAM.DataFileType_DerivedSimulated,
    SAM.attrFileFormat          : SAM.DataFileFormat_DSPACK,
    SAM.attrFileSize            : SamSize('2.55MB'),
    SAM.attrEventCount          : 10,
    SAM.attrDataTier            : 'reconstructed',
    SAM.attrFirstEvent          : 1,
    SAM.attrLastEvent           : 10,
    SAM.attrParents             : ['sim.jpsi_ee.1000evts.019-1731-c5.01'],
    SAM.attrPhysicalDatastream  : 'electron',
    SAM.attrProcessId           : 929,
    SAM.attrRunType             : 'monte carlo',
    SAM.attrRunNumber           : 134,
    SAM.attrWorkGroup           : 'dzero',
    })
ExampleMetadataList.append(sampleDerivedSimulatedFile)
