#!/usr/bin/env python
import getopt,sys,os
from FBS_API import *

script = "sam_fbsng_handler.py"
ACTION_JOB_SUBMIT = "job_submit"
ACTION_JOB_LOOKUP = "job_lookup"
ACTION_JOB_KILL = "job_kill"
InfoPfx  = "INFO   :"
ErrPfx  =  "ERROR  :"
WarnPfx =  "WARNING:"

job_state_mapping = {
                       'running'  :  "active" ,
		       'pending'  :  'pending',
                       # consider job active till it disappears from job listing  
		       'ended'    :  'active',
                    }


# list of actions
possible_actions = [ACTION_JOB_SUBMIT,ACTION_JOB_LOOKUP,ACTION_JOB_KILL]

ValidOpts = {
              'executable='  : "The excutable to be launched ( name may include full path )" ,
              'arguments='   : "List of arguments to be passed to the executable (enclosed in quotes)" ,
              'project='     : "Name of the batch job (If this violates fbsng naming rules then a name will be derived from this)" ,
              'stdout='      : "Name of stdout file ( name may include full path )", 	      
              'stderr='      : "Name of stderr file ( name may include full path )", 	   
              'queue='       : "Queue to be accessed",
              'local-job-id=': "Id of a batch job (Useful wheen killing or querying individual jobs	)",
            }  

def getstrTime():
   import time
   return time.strftime("%a %D %T",time.localtime())
 
def printInfo(stringToPrint):
   print >>sys.stdout, "%s: %s: %s %s" %(script,getstrTime(),InfoPfx,stringToPrint)
 
def printError(stringToPrint):
   print >>sys.stderr, "%s: %s: %s %s" %(script,getstrTime(),ErrPfx,stringToPrint)
 
def printWarning(stringToPrint):
   print >>sys.stderr, "%s: %s: %s %s" %(script,getstrTime(),WarnPfx,stringToPrint)

def Usage():
   printError("%s Usage is" %script)
   printError("%s <possible_actions> <arguments>" %script)
   printError("Possible_actions are:\n" )
   print "\t\t\t\t", possible_actions, "\n"
   printError("Valid options\n")
   for k in ValidOpts.keys():
      print("\t\t\t\t--%s%s "%(k,ValidOpts[k]))

def getClientObject():
   try :
      fbs_client_obj = FBSClient()
   except :
      printError("Could not instantiate FBS Client Object")
      printError(sys.exc_info()[2])      
      sys.exit(1)
   return fbs_client_obj   

def getJobDescObj():
   try :
      job_desc_obj = FBSJobDesc()
   except :
      printError("Could not instantiate Job description Object")
      printError(sys.exc_info()[1])      
      sys.exit(1)
   return job_desc_obj   

def getSectionDescObj(project):
   try :
      section_desc_obj = FBSSectionDesc(project)
   except :
      printError("Could not instantiate Section description Object")
      printError(sys.exc_info()[1])      
      sys.exit(1)
   return section_desc_obj   




def do_jobsubmit(project,executable,arguments,stdout,stderr,queue) :
   if len(project) == 0 or len(executable) == 0 :
      printError("Illegal name for project or executable")
      sys.exit(1)
      
   fbs_client_obj = getClientObject() 
   job_desc_obj = getJobDescObj()
   section_desc_obj = getSectionDescObj(project)
   
   # set the values for section description
   section_desc_obj.Queue = queue
   
   jjn = 0
   if os.environ.has_key('JIM_JOB_NUMBER'):
      jjn = os.environ['JIM_JOB_NUMBER']
   else :
      printWarning("JIM_JOB_NUMBER is unavailable ... using 0 instead")
      
   if not os.environ.has_key('SAMGRID_BATCH_ADAPTER_CONFIG_DIR') :
      printError("Please setup samgrid_batch_adapter first")
      sys.exit(1)
      
   import commands
   status,user = commands.getstatusoutput("whoami")
   cmd = ". /usr/local/etc/setups.sh; setup kerberos; setup fcp"
   cmd = "%s; fcp -c scp %s@samgfarm2:%s/fbs_scratch_setup.sh /tmp/fbs_scratch_setup$$.sh" %(cmd,user,os.environ['SAMGRID_BATCH_ADAPTER_CONFIG_DIR'])
   cmd = "%s;/tmp/fbs_scratch_setup$$.sh --jim-job-number=%s --executable=%s --arguments='%s' --stdout=%s --stderr=%s"\
          %(cmd,jjn,executable,arguments,stdout,stderr)
   cmd = "%s;rm -f /tmp/fbs_scratch_setup$$.sh" %cmd	  

   section_desc_obj.Exec = '/bin/sh -c "%s"' %cmd
   section_desc_obj.PerProcRsrc = {'disk':10}

   # debug - this will fail xml update in grid submission
   #print section_desc_obj

   # populate job_desc_obj
   job_desc_obj.addSection(section_desc_obj)
   
   #submit job
   try :
      status,text = fbs_client_obj.submitJob(job_desc_obj)
   except :
      printError("Error while submitting job")
      printError(sys.exc_info()[1])      
      sys.exit(1)   

   print "%s" %text
# ============   End of do_jobsubmit	 

def getJobsForUser(user):
   if len(user) == 0:
      printError("Username cannot be of 0 length")
      sys.exit(1) 
 
   # get the list of jobs by "user" 
   return getClientObject().getJobList(user)
# ============   End of getJobsForUser   
	 
def getJobsForProject(project,returnList=0):
   if len(project) == 0:
      printError("Project name cannot be of 0 length")
      sys.exit(1)

   fbs_client_obj = getClientObject()   
   
   import commands
   status,user = commands.getstatusoutput("whoami")
   jobList = getJobsForUser(user) 
   
   # there are 2 states that we handle pending running
   isActive = 0
   projectJobList = []
   
   for job in jobList:
      job_info = fbs_client_obj.getJob(job) 
      if project in job_info.sections():
         if returnList == 0:
            isActive = 1
            try :
               mapped_state = job_state_mapping['%s' %job_info.State]
	       print "JobId=%s Status=%s" %( job,mapped_state)
            except :
               # unknown state"   
               print >>sys.stderr, "sam_fbsng_handler: Unknown state : %s" % job_info.State
	       pass 
         else :
	    projectJobList.append(job)  

   if not isActive and not returnList:
      print "JobId=UNDEFINED Status=done"
   
   if returnList : return projectJobList   
      
      
# ============   End of getJobsForProject      

def do_joblookup(project) :
   if len(project) == 0 :
      printError("You must specify either project name for lookup")
      sys.exit(1)

   getJobsForProject(project) 

# ============   End of do_joblookup
   
                  
def do_jobkill(project,ljid) :
   if len(project) == 0 and  len(ljid) == 0:
      printError("You must specify either project or local job id for job_kill")
      sys.exit(1)

   fbs_client_obj = getClientObject() 

   if len(ljid) != 0 :
      # kill just the local job
      sts,reason = fbs_client_obj.killJob(ljid)
      if not sts :
         print >>sys.stderr, "sam_fbsng_handler: Failed to kill job %s" % ljid
   else :
      # get job list for project  
      jobList = getJobsForProject(project,1) 
      for job in jobList :
         sts,reason = fbs_client_obj.killJob(job)
         if not sts :
            print >>sys.stderr, "sam_fbsng_handler: Failed to kill job %s" % job 
        
# ============   End of do_jobkill

def main (args):
   if len(args) < 3:
      Usage()
      sys.exit(1) 
   action    = args[1]
   arguments = args[2:]     

   if not action in possible_actions:
      printError("Illegal action specified - %s\n" %action)
      Usage()
      sys.exit(2)    
   
   try :
      arg_list,sink = getopt.getopt(arguments,"",ValidOpts.keys())
   except getopt.GetoptError:
      printError("Illegal option: %s\n" % sys.exc_info()[1])
      Usage()
      sys.exit(2)

   # Initialize all variables
   project = arguments = executable = stderr = stdout = ljid = queue = ""
   import string    

   for o,a in arg_list:
      if o == "--executable" :
         executable = a
	 if string.find(executable,"/") == -1:
	    executable = "%s/%s" %(os.getcwd(),executable)

      if o == "--project" :
         project = a 
      if o == "--arguments":
         arguments = a
	 
      if o == "--stdout" :
         stdout = a
         if string.find(stdout,"/") == -1:
	    stdout = "%s/%s" %(os.getcwd(),stdout)
  
      if o == "--stderr" :
         stderr = a
         if string.find(stderr,"/") == -1:
	    stderr = "%s/%s" %(os.getcwd(),stderr)	    

      if o == "--local-job-id" :
         ljid = a

      if o == "--queue" :
         queue = a

      
   # filter out project name otherwise the scratch path
   # turns out to be too long and d0 code starts crashing
   import re
   project = re.sub('[a-z].*\.[a-z]*_',"",project)
   project = re.sub('_',"",project)
 
   # Set defaults
   if len(queue) == 0 : queue = "SAMGrid"

   if action == ACTION_JOB_SUBMIT :
      do_jobsubmit(project,executable,arguments,stdout,stderr,queue)
   elif action == ACTION_JOB_LOOKUP :
      do_joblookup(project)
   elif action == ACTION_JOB_KILL :   
      do_jobkill(project,ljid)
   else :
      printError("Action \"%s\" is not handled" %action)

if __name__ == '__main__':
    sys.exit(main(sys.argv))
