#!/bin/sh

prog=`basename $0`

SCRATCH_AREA="/scratch"
# This script must not output too much into stdout when it's called
# for job lookup!

die () { echo "$prog: $@" >&2; exit 1; }
usage () { die Possible actions: `egrep '^[a-z].*\)$' $0 | sed 's?)??g'`; }
get_value () { echo $1 | sed "s?^--[a-z|-]*=??g"; }
max_job_name_len=10
PATH=/usr/local/bin/:$PATH

#Arguments:
action=$1
shift
[ -n "$action" ] || usage

while [ -n "$1" ]; do
   case $1 in
   --executable=*)
      #This must not be a full path
      executable=`get_value "$1"`
      ;;

   --arguments=*)
      arguments=`get_value "$1"`
      ;;

   --project=*)
      project=`get_value "$1"`
      #modify the grid job id supplied to make it  10 char in length also strip away the special character as the 1st of the last 10 character 
      #might be a special character and that will cause qsub to fail
      #quickfix: use the last 10 chars (that will be unique)

      project=`echo $project | sed -e 's/_//g' -e 's/\.//g' -e 's/-//g'`
      len=`echo $project | wc -c|sed -e "s/ //g"`	

      if [ $len -gt $max_job_name_len ]; then
         len=`expr $len - $max_job_name_len`
         project=`echo $project | cut -c $len-`		
      fi
      #if the 1st character is a digit make it blank otherwise pbs barks
      project=`echo $project |  sed -e's/^[0-9]/Z/g'`
      ;;

   --stdout=*)
      stdout=`get_value "$1"`
      ;;

   --stderr=*)
      stderr=`get_value "$1"`
      ;;

   --queue=*)
      queue=`get_value "$1"`
      ;;
  
   --local-job-id=*)
      local_job_id=`get_value "$1"`
      ;;

   *)
      usage
   esac
   shift
done


# In PBS if job has finished rc=153 and if it is running rc=0
# This is ok even for qdel (i.e if we try to delete a job that has already finished it is not a fatal error)
# All other rcs are assumed to be Error conditions


#include full paths to all pbs commands
case $action in
job_submit|project_submit|consumer_submit)
   [ -n "$executable" -a -n "$project" ] || \
   die "specify project and executable when submitting job"
        REMOTE_BOOTSTRAP="/tmp/$executable.jim$$"
        SETUP_SCRIPT="/tmp/pbs_scratch_setup_$$.sh"
   
   [ -n "$SAMGRID_BATCH_ADAPTER_CONFIG_DIR" ] || die "setup samgrid_batch_adapter first!"
   
   cmd="/bin/echo \"${SETUP_SCRIPT} --executable=$REMOTE_BOOTSTRAP --arguments=\\\"$arguments\\\"
        --scratch=$SCRATCH_AREA ; rm -rf $SETUP_SCRIPT \" | qsub -N $project -v SAM_STATION,JIM_JOB_NUMBER 
       -W stagein=$REMOTE_BOOTSTRAP@`hostname`:`pwd`/$executable,$SETUP_SCRIPT@`hostname`:$SAMGRID_BATCH_ADAPTER_CONFIG_DIR/pbs_scratch_setup.sh"

   [ -n "$stdout" ] && cmd="$cmd -o $stdout"
   [ -n "$stderr" ] && cmd="$cmd -e $stderr"
   [ -n "$queue" ] && cmd="$cmd -q $queue"  	

   eval $cmd	

   if [ $? != 0 ];then
      echo "$prog:Job Submission failed" >&2
      exit 1
   fi

   ;;


job_lookup|consumer_lookup|project_lookup)
   out=`qstat` 
   if [ $? != 0 ];then
      die "$prog: JOB LOOKUP FAILED: ${out}"
   fi

   if [ -n "${local_job_id}" ]; then
      #For this purpose only, truncate the local job ID
      local_job_id=`echo $local_job_id | cut -f1 -d.`
      cmd="echo \"\$out\" | egrep ^$local_job_id"
   elif [ -n "${project}" ]; then 
      cmd="echo \"\$out\" | fgrep $project"
   else
      die "Specify local job ID or project name for job lookup"
   fi
        
   eval $cmd | awk  'BEGIN { statusMap["T"]="suspended"; \
                             statusMap["H"]="suspended"; \
                             statusMap["R"]="active"; \
                             statusMap["Q"]="pending"; \
                             statusMap["W"]="failed";\
                             statusMap["E"]="active";\
                             notFound=1; fmt="JobId=%s Status=%s\n";} \
                             {  notFound=0; printf fmt, $1, statusMap[$5]; }\
                     END { if ( notFound ) printf fmt, "UNDEFINED", "done" }'

   ;;

 
job_kill|consumer_kill|project_kill)
   out=`qstat` 
   if [ $? != 0 ];then
      die "$prog: JOB KILL FAILED: ${out}"
   fi

   if [ -n "$project" ]; then
      local_job_id=`echo "$out" | fgrep $project | awk '{print $1}'`
   elif [ -z "${local_job_id}" ]; then
      die "Specify local job ID or project name for job kill"
   fi
   
   if [ -n "$local_job_id" ]; then
     qdel $local_job_id
     if [ $? != 0 ]; then
       #sometime the output of qstat cuts the jobid
       for ljid in $local_job_id; do
         real_local_job_id="$real_local_job_id `qstat -f | grep $ljid | sed -e 's?Job Id: ??g'`"
       done
       qdel $real_local_job_id

     fi
   fi
   ;;  
  
*)
   	usage
esac
