#!/usr/bin/env sampy
################################################################################
# Project   : SAM
# Package : sam_cp
# $Id: sam_cp.py,v
# 1.29 2001/06/04 21:39:09 lauri Exp $
#
# This work is part of a development project, called SAM, which consists of a
# number of coordinated packages each named sam_xxxx .
#
# Notice of authorship, copyright status,  and terms and conditions, should
# the software eventually become available for use outside Fermilab, can be
# found in the README and LICENCE files in the top level directory of the main
# sam package.
################################################################################


import sys
import copy
import string
from types import *

UNKNOWN_DOMAIN = "UNKNOWN_DOMAIN"

def fakeLogger(aString):
    sys.stdout.write("%s\n" % aString)

class SamCpDomainHandler:
    def __init__(self, configuredDomainDictionary = {}, log=fakeLogger ):
        # incoming dictionary of configured domains:
        self.availableCpPerDomain = configuredDomainDictionary

        # now cludge a "catch-all" key into this dictionary for
        # nodes which are NOT configured!
        self.availableCpPerDomain[UNKNOWN_DOMAIN] = []

        # now build the list of known domains.
        self.knownDomains = configuredDomainDictionary.keys()
        self.dbgprint = log

        self.dbgprint("SamCpDomainHandler: knownDomains = %s" % self.knownDomains)
        for domain in self.knownDomains:
            self.dbgprint("SamCpDomainHandler:   cpProtocols[%s] = %s" % (domain, self.availableCpPerDomain[domain]))
        
        


    # input: a real live parsed domain string
    # output: the best match within the domainDictionary of all known domains.
    def samCpDomain(self, domain):
        bestMatch = ""
        if ( domain in self.knownDomains ):
            bestMatch = domain
        else:
            for possibleMatch in self.knownDomains:
                if ( string.find(domain, possibleMatch) != -1 ):
                    if ( len(possibleMatch) > len(bestMatch) ):
                        bestMatch = possibleMatch

        if ( bestMatch == "" ):
            bestMatch = UNKNOWN_DOMAIN

        self.dbgprint("incoming domain = %s .... best match in map = %s" % (domain, bestMatch))
        return bestMatch


        
    def getAvailableProtocols(self, listOfInputDomains):

        # widest possibility is all of the ones listed first.
        domain = listOfInputDomains[0]
        availableProtocols = self.availableCpPerDomain[domain]

        # now start filtering out what does NOT appear for other
        # domains.
        for i in range(len(availableProtocols)):
            for domain in listOfInputDomains[1:]:
                if availableProtocols[i] not in self.availableCpPerDomain[domain]:
                    availableProtocols[i] = None

        # now remove all of the None values from availableProtocols:
        while ( None in availableProtocols ):
            del( availableProtocols[availableProtocols.index(None)] )

        self.dbgprint("getAvailableProtocols(%s) -> %s" % (listOfInputDomains, availableProtocols))
        return availableProtocols


if __name__ == "__main__":
    configuredDomains = { 'fnal.gov' : ['sam_bbftp', 'sam_kerberos_rcp', 'sam_local_cp'],
                          'enstore'  : ['sam_encp',],
                          'nikhef'   : ['sam_bbftp',],
                          }
    domainHandler = SamCpDomainHandler(configuredDomains)
    for l in ( ('d0ora3.fnal.gov', 'd0ora3.fnal.gov', 'd0ora3.fnal.gov'),
                  ('d0ora3.fnal.gov', 'd0mino.fnal.gov', 'd0ora3.fnal.gov'),
                  ('d0ora3.fnal.gov', 'd0ora3.fnal.gov', 'd0mino.fnal.gov'),
                  ('d0mino.fnal.gov', 'd0ora3.fnal.gov', 'd0ora3.fnal.gov'),
                  ('d0ora3.fnal.gov', 'd0mino.fnal.gov', 'uta.hep.nikhef') ):

        newList = []
        for item in l:
            newList.append( domainHandler.samCpDomain(item) )
                            
        availableProtocols = domainHandler.getAvailableProtocols(newList)
        print ("list = %s,\n      bestDomainMatch = %s,\n       protocols = %s" % (l, newList, availableProtocols))
