#!/usr/bin/env sampy
import sys
import string

class SamCpException(Exception): pass

class SamCpFailure(SamCpException):
    """
    Generic failure message
    """
    def __init__(self, msg):
        self.args = msg

class SamCpFileTransferFailure(SamCpException):
    """
    File transfer failure message
    """
    def __init__(self, msg):
        self.args = msg

class SamCpPostProcessFailure(SamCpException):
    """
    Post processing failure message
    """
    def __init__(self, msg):
        self.args = msg

class SamCpExecvpFailure(SamCpException):
    """
    Failure class when we tried to Execvp a command,
    but couldn't get there from here.
    """
    def __init__(self, cmdAttempted, msg):
        self.args = "FAILED to '%s':\nBECAUSE: %s" % (cmdAttempted, msg)

class SamCpUnrecognizedProtocol(SamCpException):
    """
    Selected protocol is not known
    """
    def __init__(self, protocolName):
        self.args = "My configuration does not include a python class for protocol '%s'" % protocolName

class SamCpMethodNotImplementedException(SamCpException):
    """
    Coding error
    """
    def __init__(self, *args):
        callingFrame = sys._getframe(1)
        calledFunction = callingFrame.f_code.co_name

        # figure out who called us:
        callingClass = callingFrame.f_locals.get('self').__class__.__name__

        msg = "INTERNAL ERROR: class '%s'  did not implement the '%s' method!" % \
              (callingClass, calledFunction)
        SamCpException.__init__(self, msg)

    
