#!/usr/bin/env sampy
###############################################################################
# Project : SAM
# Package : sam_cp
# $Id
# $
#
# This work is part of a development project, called SAM, which consists of a
# number of coordinated packages each named sam_xxxx .
#
# Notice of authorship, copyright status,  and terms and conditions, should
# the software eventually become available for use outside Fermilab, can be
# found in the README and LICENCE files in the top level directory of the main
# sam package.
###############################################################################

import types
import sys
import string

import SamCpExceptions

###############################################################################
# Constants.

PRE_PROCESS_STD_OUT_KEY_ = "PreProcessStdOut"
PRE_PROCESS_STD_ERR_KEY_ = "PreProcessStdErr"
PRE_PROCESS_EXIT_STATUS_KEY_ = "PreProcessExitStatus"

FILE_TRANSFER_STD_OUT_KEY_ = "FileTransferStdOut"
FILE_TRANSFER_STD_ERR_KEY_ = "FileTransferStdErr"
FILE_TRANSFER_EXIT_STATUS_KEY_ = "FileTransferExitStatus"

POST_PROCESS_STD_OUT_KEY_ = "PostProcessStdOut"
POST_PROCESS_STD_ERR_KEY_ = "PostProcessStdErr"
POST_PROCESS_EXIT_STATUS_KEY_ = "PostProcessExitStatus"

ADDITIONAL_INFO_KEY_ = "AdditionalInfo"

ENSTORE_CRC_VALUE_KEY_ = "CRCValue"
ENSTORE_VOLUME_LABEL_KEY_ = "VolumeLabel"
ENSTORE_VOLUME_OFFSET_KEY_ = "VolumeOffset"

STD_OUT_STREAM_ = sys.stdout
STD_ERR_STREAM_ = sys.stderr
DEFAULT_STREAM_ = sys.stdout

UNKNOWN_DCACHE_VOLUME_OFFSET_ = "UnknownDcacheVolumeOffset"
UNKNOWN_DCACHE_VOLUME_LABEL_ = "UnknownDcacheVolumeLabel"
UNKNOWN_DCACHE_CRC_VALUE_ = "UnknownDcacheCRCValue"


###############################################################################
# Exception classes.

class SamCpOutputHandlerException(SamCpExceptions.SamCpException):

  def __init__(self, error):
    self.args = error

###############################################################################
# Output handler class.

class SamCpOutputHandler:

  KEY_STREAM_PRIORITY_LIST_ = [ 
    (ENSTORE_CRC_VALUE_KEY_ , STD_OUT_STREAM_),
    (ENSTORE_VOLUME_LABEL_KEY_ , STD_OUT_STREAM_),
    (ENSTORE_VOLUME_OFFSET_KEY_ , STD_OUT_STREAM_),
    (PRE_PROCESS_EXIT_STATUS_KEY_ , STD_OUT_STREAM_),
    (PRE_PROCESS_STD_OUT_KEY_ , STD_OUT_STREAM_),
    (PRE_PROCESS_STD_ERR_KEY_ , STD_ERR_STREAM_),
    (FILE_TRANSFER_EXIT_STATUS_KEY_ , STD_OUT_STREAM_),
    (FILE_TRANSFER_STD_OUT_KEY_ , STD_OUT_STREAM_),
    (FILE_TRANSFER_STD_ERR_KEY_ , STD_ERR_STREAM_),
    (POST_PROCESS_EXIT_STATUS_KEY_ , STD_OUT_STREAM_),
    (POST_PROCESS_STD_OUT_KEY_ , STD_OUT_STREAM_),
    (POST_PROCESS_STD_ERR_KEY_ , STD_ERR_STREAM_),
    (ADDITIONAL_INFO_KEY_ , STD_OUT_STREAM_),
    ]


  def __init__(self, outputDict = {}):
    if type(outputDict) != types.DictType:
      err = "%s: Invalid argument type." % (self.__class__)
      raise SamCpOutputHandlerException(err)
    self._outputDict = outputDict



  def add(self, outputDict):
    if type(outputDict) != types.DictType:
      err = "%s: Invalid argument type." % (self.__class__)
      raise SamCpOutputHandlerException(err)
    
    for key in outputDict.keys():
      self._outputDict[key] = outputDict[key]

  def writeOutput(self):
    # Write special dictionary items into their corresponding streams.
    writtenKeys = []
    for item in SamCpOutputHandler.KEY_STREAM_PRIORITY_LIST_:
      key = item[0]
      stream = DEFAULT_STREAM_
      
      if len(item):
        stream = item[1]

      if self._outputDict.has_key(key):
        itemStr = "%s" % (self._outputDict[key])
        if string.count(itemStr, "\n"):
          msg = "\n%s:\n%s\n" % (key, itemStr)
        else:
          msg = "\n%s: %s\n" % (key, itemStr)
        stream.write(msg)
        writtenKeys.append(key)

    # Write other stuff into the default stream.
    otherItemDict = {}
    for key in self._outputDict.keys():
      
      if not writtenKeys.count(key):
        otherItemDict[key] = self._outputDict[key]

    if len(otherItemDict):
      msg = "\n%s:\n%s\n" % (ADDITIONAL_INFO_KEY_, otherItemDict)
      DEFAULT_STREAM_.write(msg)
        
    STD_OUT_STREAM_.flush()
    STD_ERR_STREAM_.flush()
    DEFAULT_STREAM_.flush()


###############################################################################
# Unit tests.

if __name__ == "__main__":

  try:
    oh = SamCpOutputHandler({
      'FileTransferExitStatus' : 1,
      'FileTransferStdOut' : "not good",
      'FileTransferStdErr' : "oupsss!",
      'OtherStuff' : "what can you do, this stuff does not work",
      })
    oh.writeOutput()
  except SamCpOutputHandlerException, ex:
    print ex
