## This script will configure fcp by settuon $SAM_FCP_DIR as 
## FCP_ROOT and then copying/creating files under this dir.
## This needs to be done otherwise sam_fcp cannot be declared 
## current because fcp is one of the dependencies and "setup fcp"
## fails if this is not done

## Instructions for fcp installation - http://www-isd.fnal.gov/fcp/Install.txt

#!/bin/sh

prog=`basename $0`
sep="=========================================================================================="
die_str1="It is likely that sam_fcp installation will fail.Install and tailor fcp  and try again" 
die_str2="Installation instructions for fcp available at http://www-isd.fnal.gov/fcp/Install.txt"

echoInfo()    { echo "$prog: INFO   : $*"; }
echoWarning() { echo "$prog: WARNING: $*"; }
echoError()   { echo "$prog: ERROR  : $*"; }
die (){ echoError "$*";echoError $die_str1; echoError $die_str2; echo ""; echo $sep; exit 1; }

synchronize ()
{
   if [ -z "${UPS_THIS_DB}" ]; then
      echoWarning "Could not find UPS_THIS_DB" 
      echoWarning "Will Create a new cfg file"
      return   
   fi   

   link="${UPS_THIS_DB}/sam_fcp/Symlinks/current"

   if [ -L $link ]; then
      curr_ver_link=`ls -o $link | sed -e 's/.*> //g'`
      curr_ver_link="${UPS_THIS_DB}/sam_fcp/Symlinks/$curr_ver_link"
   else
      echoWarning "$link not a valid link" 
      return 
   fi   

   if [ -L ${curr_ver_link} ]; then
      prev_prod_dir=`ls -o ${curr_ver_link} | sed -e 's/.*> //g'`
      old_cfg_file="${prev_prod_dir}/fcp/cfg/fcp.cfg"
   else
      echoWarning "${curr_ver_link} not a valid link" 
      return
   fi   

   esc_dir_path=`echo $sam_fcp_dir | sed -e 's/\//\\\\\//g'`
   if [ -r $old_cfg_file ]; then
      cat $old_cfg_file | sed -e "s/log_file.*/log_file = $esc_dir_path\/fcp\/log\/fcp_log/g" > $sam_fcp_dir/fcp/cfg/fcp.cfg 
      rc=$?
   else
      echoWarning "Older cfg file $old_cfg_file not found"
      return
   fi   

   if [ $rc != 0 ]; then
      echoWarning "Copy Failed ..."
      return 
   fi   

   echoInfo "Copied file $old_cfg_file to $sam_fcp_dir/fcp/cfg/fcp.cfg successfully"
}



echo $sep
echo ""
echoInfo "Configuring fcp ..." 
sam_fcp_dir=$1

[ -n "$sam_fcp_dir" ] || die "sam_fcp_dir not known (needs to be passed as an arg. to this script)"
[ -d "$sam_fcp_dir" ] || die "\"$sam_fcp_dir\" doesnot exist." 

cd $sam_fcp_dir || die "Couldnot change dir to $sam_fcp_dir"

mkdir -p fcp || die "Couldnot create dir to $sam_fcp_dir/fcp"
cd fcp || die "Couldnot change dir to $sam_fcp_dir/fcp"

#mkdir -p bin || die "Couldnot create dir to $sam_fcp_dir/fcp/bin"
mkdir -p log || die "Couldnot create dir to $sam_fcp_dir/fcp/log"
mkdir -p cfg || die "Couldnot create dir to $sam_fcp_dir/fcp/cfg"

# save ups db var. We'll loose it when we . ups setup ups
UPS_THIS_DB_SAV=$UPS_THIS_DB


## tailor fcp
echoInfo "Setting FCP_ROOT to $sam_fcp_dir/fcp"
cmd="ups tailor fcp -O $sam_fcp_dir/fcp"
echoInfo "Executing command \"$cmd\""
eval $cmd || die "Could not tailor fcp"

. `ups setup ups`
setup fcp || die "Could not setup fcp"



## We do not rely on this file to start the server
#Create start_fcp.sh from our template. 
# we don't want to copy it from FCP_DIR as we want to run fcp under
# server_run
#echoInfo "Creating file $sam_fcp_dir/fcp/bin/start_fcp.sh"
#cat << EOF > $sam_fcp_dir/fcp/bin/start_fcp.sh
#. \`ups setup ups\`
#(while true ;do
#   (setup fcp; fcpd </dev/null >/dev/null 2>/dev/null )
#    sleep 60
#done) &
#EOF
#if [ $? != 0 ]; then
#   die "Could not create $sam_fcp_dir/fcp/bin/start_fcp.sh"
#fi
# give exe permission to start_fcp.sh
#echoInfo "Making file $sam_fcp_dir/fcp/bin/start_fcp.sh executable"
#chmod +x $sam_fcp_dir/fcp/bin/start_fcp.sh || \
#echoWarning "Could not make start_fcp.sh executable. Execute\
# \"chmod +x $sam_fcp_dir/fcp/bin/start_fcp.sh\" from command line"



## check if fcp.cfg file is present in previous version
## if so copy that else create a new one

# restore UBS_THIS_DB variable
UPS_THIS_DB=$UPS_THIS_DB_SAV
out=`synchronize`
echo "$out"
test=`echo $out | fgrep WARNING`

if [ -n "$test" ]; then
   echoWarning "Syncronization failed"
   echoInfo "Creating new cfg file $sam_fcp_dir/fcp/cfg/fcp.cfg"   

cat << EOF > $sam_fcp_dir/fcp/cfg/fcp.cfg
%set fcp
port = 7788
max_xfers = 15          # downloads + uploads < 15
timeout = 5             # minutes
log_file = $sam_fcp_dir/fcp/log/fcp_log
%set logical_volumes
#vol1 = /stage1/vol1 /stage2/vol1 /stage3/vol1
EOF
fi


if [ $? != 0 ]; then
   die "Could not create $sam_fcp_dir/fcp/cfg/fcp.cfg"
fi


## Make $sam_fcp_dir/fcp group writable
echoInfo "Making $sam_fcp_dir/fcp group writable"
chmod g+w -R $sam_fcp_dir/fcp/ ||\
echoWarning "Couldnot make $sam_fcp_dir/fcp group writable. Execute\
\"chmod g+w -R $sam_fcp_dir/fcp\" from command line"

echoInfo "To modify configuration edit file $sam_fcp_dir/fcp/cfg/fcp.cfg"
echoInfo "Successfully finished configuring fcp ..."

echo ""
echo $sep


