#!/bin/sh

prog=`basename $0`
echoInfo()    { echo "$prog: `date "+%a %D %T"`: INFO   : $*"; }
echoWarning() { echo "$prog: `date "+%a %D %T"`: WARNING: $*"; }
echoError()   { echo "$prog: `date "+%a %D %T"`: ERROR  : $*"; }

die (){ echoError "$*"; exit 1; }

remove_file () { chmod ug+w $1; rm -rf $1; }

#passed as arguents: the .cfg files that are present in the /tmp dir.
start_queues(){
	# This is when our configuration is still obsolete and no fcp.cfg files are created.
	if [ -z "$1" ]; then
		echo -n "Starting FCP. Activating default Queue....."
		$FCP_DIR/bin/fcpd </dev/null >/dev/null 2>/dev/null &
                pid=$!
		echo "$pid" > $TMPDIR/fcp_default.pid
		echo "Started"
	fi 
	while [ -n "$1" ]; do
		# For every fcp config file start a queue
		FCP_CONFIG=$1
		# Get queue name from the config file name
		#echo "Reading Configuration from $FCP_CONFIG"
		queue_name=`echo $FCP_CONFIG | cut -f1 -d. | sed -e 's/.*fcp_//g'`
		port=`cat $FCP_CONFIG | grep port | cut -f2 -d=`
		# Start the FCP Daemon
		$FCP_DIR/bin/fcpd </dev/null >/dev/null 2>/dev/null &
		pid=$!
		# This sleep is essential to make sure that the daemon is still running
		# If the daemon crashes due to a server already running at the port, it
		# wont disappear from the "ps ax" list immediately. So we wait a second
		# to make sure that the daemon is still running or we print the alternative
		sleep 1
		tmp=`ps $pid | grep -v PID`
		# There was already an active queue on the port
		if [ -z "$tmp" ]; then
			echoInfo "Queue already active on $port"
		# We just started running on the port.
		else
			echoInfo "Started Queue $queue_name"
			echo "$pid" > $TMPDIR/fcp_$queue_name.pid
		fi
		shift
	done
}

stop_queues(){
	while [ -n "$1" ]; do
		echo -n "Stopping FCP Server Daemon(pid:$1)......."
		kill -9 $1
		echo "Stopped"
		shift
	done
}

sam_fcp_dir=$1

if [ "z$TMPDIR" = "z" ]; then
   export TMPDIR="/tmp"
fi

[ -n "$sam_fcp_dir" ] || die "sam_fcp_dir not known (needs to be passed as an arg. to this script)"
[ -n "$FCP_DIR" ]     || die "FCP_DIR env variable not set. Setup fcp first"

# looks for the .cfg files that are present in the /tmp dir. Then counts the number of .cfg files
cfg_files=`jim_config_manager_cmd.py gcs -n sam_fcp | python $SAM_FCP_DIR/ups/parse.py`
if [ $? != 0 ]; then 
  echo "\n \n"
  echo "CRITICAL ERROR:"
  die "Please check the sam_fcp configuration. Have you tailored it correctly"
fi
file_ct=`echo $cfg_files | wc -w`

# prints the pid of the fcpd that is running. then calculates the number of fcpd that are running
cmd="ps -au `whoami` | fgrep fcpd | grep -v fcpd.py | grep -v grep | awk '{print \$1}'"
pid=`eval $cmd`
pid_ct=`echo $pid | wc -w`

# Start Queues under different conditions

# If all queues are currently active
if [ $pid_ct -eq $file_ct ]; then
	if [ $pid_ct -eq 0 ] ; then
		echoInfo "Activating Default Queue"
		start_queues
	else
		echoInfo "$pid_ct fcp queues active. No more can be activated...."
		exit;
	fi
# If number of active queues is less than configuration, restart the rest.
elif [ $pid_ct -lt $file_ct ]; then
	echoInfo "$pid_ct queues active. Activating Remaining Queues...."
	start_queues $cfg_files
# If there are more active queues than configuration
else
	# This is when configuration is obsolete and so only one queue is present and running
	if [ $file_ct -eq "0" ]; then
		echoInfo "Default Queue Active. No more can be activated....."
	# This is when we change configuration and the no. of queues running is more than configuration
	else
		echoInfo "Configuration Changed. Restarting the Daemon...."
		stop_queues $pid
		start_queues $cfg_files
	fi
fi

#Keep the process in foreground
cmd="ps -au `whoami` | fgrep fcpd | grep -v fcpd.py | grep -v grep | awk '{print \$1}'"
pid=`eval $cmd`
while true; do
   for a_pid in $pid
   do
      ps $a_pid 1>/dev/null 2>/dev/null
      if [ $? = 0 ]; then
         echoInfo "fcp server process $a_pid still alive"
      else
         echoInfo "fcp server process $a_pid died"
         exit 1
      fi
   done
   sleep 3577
done

